/* MakeDoc - Aaron Bell 1998                       */
/* Turn tagged comments into HTML.                 */
/* Note nested tags are not supported!             */

/* Update: July 98. D/d tags, more compact output. */
/*
Supported tags:
   n - (where n is a number 1-9) text becomes a size n header
   p - new Paragraph
   x - eXample
   u - begin Unformatted list
   o - begin Ordered list
   l - List item
   D - definition (item)
   d - definition (explanation)
*/

parse arg infile outfile .

header.0 = 3
header.1 = '<html>'
header.2 = '<body bgcolor="#ffffff">'
header.3 = '<font size=2>'

footer.0 = 3
footer.1 = '</font>'
footer.2 = '</body>'
footer.3 = '</html>'

OPEN_TAG = ''
cr = d2c(13)

IF outfile = '' THEN DO
  say 'Usage: MAKEDOC <input file> <HTML output file>'
  say 'by Aaron Bell June 98. Turns tagged comments into HTML.'
  exit 1
END
IF stream(outfile,'c','QUERY EXISTS') <> '' THEN DO
  say 'Output file already exists! Aborted.'
  exit 1
END
IF stream(infile,'c','QUERY EXISTS') = '' THEN DO
  say "Input file '"infile"'not found."
  exit 1
END

signal on syntax

rc = stream(infile,'c','OPEN READ')
rc = stream(outfile,'c','OPEN WRITE')

say "  MakeDoc, Aaron Bell 1998. Converting '"infile"'"
DO i = 1 TO header.0
  call writeline(header.i)
END
linecount = 0

DO FOREVER WHILE (lines(infile) > 0)
  call readline
  rc = charout(stdout,cr'Processing line' linecount'...')
  IF left(line,2) <> '//' THEN DO
    call check_open_tags
    DO i = 1 TO footer.0
      call writeline(footer.i)
    END
    call stop('No more comments, done.')
  END

  parse var line '//' data

  IF left(data,1) <> ' ' THEN call process(data)
  ELSE call writeline(data)
END
call stop('End of file,' linecount 'lines read.')

syntax:
call stop('Ctrl-C, aborted.')

/*---------------------------------------------------------------------------*/
stop: parse arg message
 say
 rc = stream(infile,'c','CLOSE')
 rc = stream(outfile,'c','CLOSE')
 say 'Stopped:' message
 say
 exit 0
return
/*---------------------------------------------------------------------------*/
readline:
 IF (lines(infile) > 0) THEN DO
   line = linein(infile)
   linecount = linecount + 1
 END
 ELSE call stop('readline, unexpected end of file.')
return
/*---------------------------------------------------------------------------*/
writeline: parse arg the_line
 IF OPEN_TAG <> 'X' THEN the_line = strip(the_line)
 rc = lineout(outfile,the_line)
 IF rc <> 0 THEN call stop('Error writing to' outfile'!')
return
/*---------------------------------------------------------------------------*/
process: parse arg the_data
 parse var the_data the_tag the_comment
 up_tag = translate(the_tag)

 IF up_tag = 'P' THEN DO
   call check_open_tags
   call writeline('<p>')
   IF the_comment <> '' THEN call writeline(the_comment)
 END
 /*-------------------------------*/
 ELSE IF datatype(up_tag,'N') = 1 THEN DO
   IF up_tag > 9 THEN call stop('tag is number greater than 9.')
   call check_open_tags
   call writeline('<h'up_tag'>')
   IF the_comment <> '' THEN call writeline(the_comment)
   OPEN_TAG = up_tag
 END
 /*-------------------------------*/
 ELSE IF up_tag = 'X' THEN DO
   call check_open_tags
   call writeline('<xmp>')
   OPEN_TAG = 'X'
   IF the_comment <> '' THEN call writeline(the_comment)
 END
 /*-------------------------------*/
 ELSE IF up_tag = 'U' THEN DO
   call check_open_tags
   call writeline('<ul>')
   OPEN_TAG = 'U'
 END
 /*-------------------------------*/
 ELSE IF up_tag = 'O' THEN DO
   call check_open_tags
   call writeline('<ol>')
   OPEN_TAG = 'O'
 END
 /*-------------------------------*/
 ELSE IF up_tag = 'L' THEN DO
   IF ( (OPEN_TAG = 'U') | (OPEN_TAG = 'O') ) THEN call writeline('<li>'the_comment)
   ELSE call stop('List item without open list')
 END
 /*-------------------------------*/
 ELSE IF the_tag = 'D' THEN DO
   call check_open_tags
   call writeline('<dt><b>'the_comment'</b>')
   OPEN_TAG = 'D'
 END
 /*-------------------------------*/
 ELSE IF the_tag = 'd' THEN DO
   IF (OPEN_TAG = 'D') THEN DO
     call writeline('<dd>'the_comment)
   END
   ELSE call stop('Definition (d) without item (D)')
 END
 /*-------------------------------*/
 ELSE call stop('Unrecognised tag: 'the_tag)
return
/*---------------------------------------------------------------------------*/
check_open_tags:
 IF OPEN_TAG = '' THEN return

 IF datatype(OPEN_TAG,'N') = 1 THEN call writeline('</h'OPEN_TAG'>')
 ELSE IF OPEN_TAG = 'X' THEN call writeline('</xmp>')
 ELSE IF OPEN_TAG = 'U' THEN call writeline('</ul>')
 ELSE IF OPEN_TAG = 'O' THEN call writeline('</ol>')
 ELSE IF OPEN_TAG = 'D' THEN OPEN_TAG = ''

 OPEN_TAG = ''
return
