//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      ascj001 - Multiple Asynchronous ECI Calls with Callback
//p
//      This application can run multiple ECI asynchronous calls with
//      callback simultaneously.
//p
//      This source file creates two classes:
//o
//l     ascj001.class ........ issues the asynchronous calls.
//l     ascjCB.class ......... handles all callbacks received.
//p
//      Prior to each ECI call the commarea is initialised to
//      18 dashes. The server returns the current date and timestamp.
//      The default CICS server program is the supplied SRVTIME
//      which is compatible on all CICS servers. If desired, a different server
//      program can be supplied as an input parameter to this application.
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Parameters can be overridden via command line.
//p
//      Issue "ascj001 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.ascj001 -j local: -calls 3 -i 2
//p
//      The sample input requests three ECI asynchronous calls to be made,
//      iterating twice. Each ECI call has an associated message qualifier
//      number which is displayed upon the call's return. Being asynchronous
//      the responses to the ECI calls do not necessarily return in the same
//      order that they were issued. The sample output demonstrates this scenario.
//p
//      Sample Output
//x
//      ascj001 - Multiple Asynchronous ECI Calls with Callback
//             -j     JGate                  -> local:
//             -r     JGate port             -> 2006
//             -list  list systems only      -> false
//             -s     CICS server            -> (not specified)
//             -a     application            -> SRVTIME
//             -calls # calls per iteration  -> 3
//             -i     # iterations           -> 2
//             -u     userid                 -> SYSAD
//             -p     password               -> SYSAD
//             -d     delay                  -> 0 seconds
//             -x     exit on ECI error      -> false
//             -!     debug                  -> false
//             -?     Display settings only
//
//      --- ascj001 started 11/06/98 14:56
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: AlsKix
//
//      #   Call               Commarea Data      Length  CICS Rc  Abend  Call OK?
//      --  -----------------  -----------------  ------  -------  -----  --------
//      1   eci_async + mq 1      -- flow -->
//      1   eci_async + mq 2      -- flow -->
//      1   eci_async + mq 3      -- flow -->
//          eci_async + mq 1   11/06/98 15:56:24  18      0        null    OK
//          eci_async + mq 2   11/06/98 15:56:25  18      0        null    OK
//          eci_async + mq 3   11/06/98 15:56:25  18      0        null    OK
//      2   eci_async + mq 1      -- flow -->
//      2   eci_async + mq 2      -- flow -->
//      2   eci_async + mq 3      -- flow -->
//          eci_async + mq 3   11/06/98 15:56:25  18      0        null    OK
//          eci_async + mq 2   11/06/98 15:56:26  18      0        null    OK
//          eci_async + mq 1   11/06/98 15:56:26  18      0        null    OK
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- ascj001 completed 11/06/98 14:56
//
//      --- Calls successful ... 6 out of 6
//      --- Iterations run ..... 2 out of 2
//p
//      History log:
//u
//l     Written : May 97 - Alan Hollingshead
//l     Updated : May 98 - uses common classes
//
//

package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class ascj001
{
   static int iReplyOK    = 0;
   static int iFlag       = 0;

   public static void main(String[] args)
   {
      int iEci_Call       = 0;
      int iIt_Count       = 0;
      int iCallsIssued    = 0;
      int iNum_Calls      = 0;
      int iCounter        = 0;
      int iReply_Count    = 0;

      String  sGetReply   = "";
      boolean bReply_Rcvd = false;

      // callback variables
      int    iMessage_Qualifier   = 99;
      Object ObjLock              = null;

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nascj001 - Multiple Asynchronous ECI Calls with Callback") ;
      System.out.println("       -j     JGate                  -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port             -> "+theseParms.sPort);
      System.out.println("       -list  list systems only      -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server            -> "+theseParms.sServer);
      System.out.println("       -a     application            -> "+theseParms.sProgname);
      System.out.println("       -calls # calls per iteration  -> "+theseParms.sCalls);
      System.out.println("       -i     # iterations           -> "+theseParms.sIterations);
      System.out.println("       -u     userid                 -> "+theseParms.sUserid);
      System.out.println("       -p     password               -> "+theseParms.sPassword);
      System.out.println("       -d     delay                  -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x     exit on ECI error      -> "+theseParms.bExitNow);
      System.out.println("       -!     debug                  -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java ascj001 <parameters>");
         System.out.println(  "  e.g. java ascj001 -s AlsKix -calls 12 -i 5");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("ascj001");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ----------------------------------------------------------------------
      // Display first iteration banner
      // ----------------------------------------------------------------------

      System.out.println("\n#   Call               Commarea Data      Length  CICS Rc  Abend  Call OK?");
      System.out.println("--  -----------------  -----------------  ------  -------  -----  --------");

      iteration_loop:
      // Iteration loop (-i parameter)
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {

         // -------------------------------------------------------------------
         // Output banner every 5 iterations
         // -------------------------------------------------------------------

         if (iIt_Count%5==0)
         {
            System.out.println("\n#   Call               Commarea Data      Length  CICS Rc  Abend  Call OK?");
            System.out.println("--  -----------------  -----------------  ------  -------  -----  --------");
         } // end-if

         // declare and instantiate object for lock
         ObjLock         = new Object();

         Thread thrMain  = Thread.currentThread();

         // Calls loop (-calls parameter)
         for (iNum_Calls=1; iNum_Calls < (theseParms.iCalls+1); iNum_Calls++)
         {
            // new commarea required for each call
            byte[] Commarea = new byte[18];
            try
            {
               // new calBack for each call
               ascjCB calBack  = new ascjCB(thrMain, ObjLock, iNum_Calls);

               ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_ASYNC,
                                                        theseParms.sServer,
                                                        theseParms.sUserid,
                                                        theseParms.sPassword,
                                                        theseParms.sProgname,
                                                        theseParms.sTransid,
                                                        Commarea,
                                                        0,
                                                        ECIRequest.ECI_NO_EXTEND,
                                                        ECIRequest.ECI_LUW_NEW,
                                                        iMessage_Qualifier,
                                                        calBack
                                                        );

               // set commarea to dashes
               for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)

               // reset commarea to dashes
               eciRequest.Commarea[iCounter]=45;

               // Convert ASCII bytes to ASCII characters
               String sDashData  = new String(eciRequest.Commarea,0);

               eciRequest.Message_Qualifier = iNum_Calls;

               String sSpace4  = "    ";
               String sGap1 = sSpace4.substring(1,(5-(String.valueOf(iIt_Count).length())));
               String sGap2 = sSpace4.substring(1,(5-(String.valueOf(iNum_Calls).length())));

               System.out.println(iIt_Count +
                                  sGap1 + "eci_async + mq " +
                                  eciRequest.Message_Qualifier +
                                  sGap2 + "   -- flow --> ");

               // flow ECI request
               iCallsIssued++;
               jgaConnection.flow(eciRequest);

               // Delay next ECI call if delay parameter specified (default=0)
               try
               {
                  Thread.sleep(theseParms.iDelay);
               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error during sleep() ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            }
            catch (Exception e)
            {
               T.ln(null,"ERROR: Caught Exception: ");
               T.ex(null,e);
               System.out.println("\n--- Error returned by JGate ---\n");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }
         // this will synchronize the last object in the iteration
         synchronized(ObjLock)
         {
            while (iFlag != (theseParms.iCalls * iIt_Count))
            {
               try
               {
                   ObjLock.wait();
               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error returned by JGate ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            }
         }
      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("ascj001", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);

   } // end-run() method

} // end-ascj001 class

// ------------------------------------------------------------------------
// Class: ascjCB (ie ascj001 callback)
// Handles all callbacks received from the Java gateway.
// ------------------------------------------------------------------------
class ascjCB implements Callbackable
{
   ECIRequest CB_eciRequest  = null;
   Thread thrToWakeUp        = null;
   Object    ObjLock         = null;
   boolean boolIsAlive       = false;
   int myCallNo              = 0;
   String sMQok              = "OK";

   public ascjCB(Thread thrWakeUp, Object ObjParm, int iCallNo)
   {
       thrToWakeUp    = thrWakeUp;
       ObjLock        = ObjParm;
       myCallNo       = iCallNo;
   }

   public void setResults(GatewayRequest gatReply)
   {
      CB_eciRequest  = (ECIRequest) gatReply;
   }

   public void run()
   {

      boolIsAlive = thrToWakeUp.isAlive();

      if (boolIsAlive)
      {
         synchronized (ObjLock)
         {
            ascj001.iFlag++;
            ObjLock.notify();

            String ComData;
            // Convert ASCII bytes to ASCII characters
            ComData = new String(CB_eciRequest.Commarea,0);

            // confirm myCallNo, which assigns Message_Qualifier in initial
            // async call, matches returned Message_Qualifier in eciRequest
            // response.
            if (myCallNo == CB_eciRequest.Message_Qualifier)
            {
               sMQok = "OK";
            }
            else
            {
               sMQok = "Wrong MQ!";
            }

            // confirm commarea has been modified by server program
            // otherwise call was not successful.
            if (ComData.startsWith("---"))
            {
               sMQok = "Commarea incorrect";
            }
            else
            {
               sMQok = "OK";
            }

            // check CICS Rc
            if (CB_eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
            {
               sMQok = "OK";
               // keep tally of successful calls
               ascj001.iReplyOK++;
            }
            else
            {
               sMQok = "Call failed";
            }

            // Display result
            String sSpace4  = "    ";
            String sGap1 = sSpace4.substring(1,(5-(String.valueOf(CB_eciRequest.Message_Qualifier).length())));
            String sSpace8  = "        ";
            String sGap2 = sSpace8.substring(1,(9-(String.valueOf(CB_eciRequest.Commarea_Length).length())));
            String sSpace9  = "         ";
            String sGap3 = sSpace9.substring(1,(10-(String.valueOf(CB_eciRequest.Cics_Rc).length())));
            String sGap4 = sSpace8.substring(1,(9-(String.valueOf(CB_eciRequest.Abend_Code).length())));

            System.out.println("    eci_async + mq " +
                        CB_eciRequest.Message_Qualifier +
                        sGap1 +
                        ComData.substring(0,CB_eciRequest.Commarea_Length-1) +
                        "  " + CB_eciRequest.Commarea_Length +
                        sGap2 +
                        CB_eciRequest.Cics_Rc +
                        sGap3 +
                        CB_eciRequest.Abend_Code +
                        sGap4 +
                        sMQok);
         }
      }
      else
      {
         // big trouble as main thread has disappeared!
         System.out.println("Main thread is dead ...");
      }
   }
}
