//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      asmj002 - Asynchronous Calls with Message Qualifiers
//p
//      Followed by:
//      ECI_GET_SPECIFIC_REPLY_WAIT or ECI_GET_SPECIFIC_REPLY loop.
//p
//      Summary: Issue 5 async calls with Message Qualifiers  5, 4, 3, 2, 1,
//      with optional 2 second delay between each one.
//      Issue ECI_GET_SPECIFIC_REPLY(_WAIT) for each Message_Qualifier
//      in order 1, 2, 3, 4, 5 checking timestamps are in reverse order.
//p
//      asmj002 is a Java program that on each requested iteration
//      issues five asynchronous calls with Message Qualifiers assigned
//      5 through 1 (in reverse order).
//p
//      This is followed either by an ECI_GET_SPECIFIC_REPLY_WAIT (if the -w
//      option is specified), or an ECI_GET_SPECIFIC_REPLY (the default) for
//      each Message Qualifier in the ascending order 1 through 5.
//p
//      The timestamps returned in the commarea should appear in descending
//      order to indicate that the eci_get_specific_reply(_wait) picked
//      up the related Message Qualifier. On fast networks the timestamps
//      may appear the same, and so the -2 option can be specified to
//      separate each asynchronous call by 2 seconds (see Options: below).
//p
//      ECI_GET_SPECIFIC_REPLY calls:
//      Up to nine ECI_GET_SPECIFIC_REPLY calls are issued to obtain
//      the required response from the server. Between each attempt to
//      receive this reply there is a one second delay to allow for network
//      traffic or a busy server. The output shows which call obtained the
//      response. For example, "eci_get_specific_reply #3 MQ 5" means that
//      the response was obtained on the third attempt for Message Qualifier 5.
//      If the result shows "eci_get_specific_reply #9" with a CICS Rc of -5,
//      this indicates that the response was not received after nine attempts.
//p
//      Prior to each asynchronous call the commarea is initialised to
//      18 dashes. The server returns the current date and timestamp.
//      The default CICS server program is the supplied SRVTIME
//      which is compatible on all CICS servers. If desired, a different server
//      program can be supplied as an input parameter to this application.
//p
//      Options:
//u
//l       -wait ... eci_get_specific_reply is the default. The -w option
//                  requests eci_get_specific_reply_wait.
//
//l       -2 ...... between each asynchronous call a 2 second delay is
//                  issued. This allows the user to see the timestamps
//                  on the replies in reverse order to confirm that the
//                  eci Message_Qualifier is being used correctly.
//
//l       -clear .. in certain circumstances (eg timeouts, or security
//                  error received on eci_get_specific_reply(_wait))
//                  there may be outstanding replies from previous runs.
//                  The -clear option issues repeated eci_get_specific_reply
//                  calls until eci_err_no_reply reports no more outstanding
//                  replies. (Alternatively stop and restart JGate).
//                  Security errors are specifically ignored.
//p
//      Warning: using the -clear and -wait options
//                  Care should be taken when running the program from multiple
//                  processes as the -clear option may clear outstanding replies
//                  which another process is waiting on (if -wait specified).
//p
//      Data conversion:
//           This program contains no data conversion for the commarea.
//           Data conversion should be carried out on the server, if required.
//p
//      Issue "asmj002 -?" for help and the default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.asmj002 -clear -2sec
//p
//      Sample Output
//x
//      asmj002 - ECI Asynchronous Calls with Message Qualifiers
//             -j     JGate                           -> hollings
//             -r     JGate port                      -> 2006
//             -list  list systems only               -> false
//             -s     CICS server                     -> (not specified)
//             -a     application                     -> SRVTIME
//             -wait  eci_get_specific_reply_wait     -> false
//             -2sec  delay between async calls       -> true
//             -clear clear outstanding replies first -> true
//             -i     # iterations                    -> 1
//             -u     userid                          -> SYSAD
//             -p     password                        -> SYSAD
//             -d     delay                           -> 0 seconds
//             -x     exit on ECI error               -> false
//             -!     debug                           -> false
//             -?     Display settings only
//
//      --- asmj002 started 12/06/98 06:48
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      Request to clear outstanding replies
//      No more outstanding replies
//
//      #  Call + Message_Qualifier       Commarea-Data      Length CICS Rc  Abend
//      -- ------------------------------ -----------------  ------ -------  -----
//      1  Asynchronous call + MsgQual 5  -----------------  18     0
//      1  Asynchronous call + MsgQual 4  -----------------  18     0
//      1  Asynchronous call + MsgQual 3  -----------------  18     0
//      1  Asynchronous call + MsgQual 2  -----------------  18     0
//      1  Asynchronous call + MsgQual 1  -----------------  18     0
//      1  eci_get_specific_reply#1 MQ 1  12/06/98 07:48:35  18     0
//      1  eci_get_specific_reply#1 MQ 2  12/06/98 07:48:33  18     0
//      1  eci_get_specific_reply#1 MQ 3  12/06/98 07:48:31  18     0
//      1  eci_get_specific_reply#1 MQ 4  12/06/98 07:48:29  18     0
//      1  eci_get_specific_reply#1 MQ 5  12/06/98 07:48:27  18     0
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- asmj002 completed 12/06/98 06:48
//
//      --- Calls successful ... 10 out of 10
//      --- Iterations run ..... 1 out of 1
//
//p
//      History log:
//u
//l     Written : Apr 97 - Alan Hollingshead
//l     Updated : Apr 98 - uses inputParms, ECIlist, setJGate classes
//

package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;


public class asmj002
{
   public static void main(String[] args)
   {
      int     iEci_Count   = 0;
      int     iCounter     = 0;
      int     iIt_Count    = 0;
      int     iMsg_Qual    = 0;
      int     iReply_Count = 0;
      int     iIterations  = 1;
      int     iReplyOK     = 0;
      int     iCallsIssued = 0;
      String  sGetReply    = "";
      boolean bReply_Rcvd  = false;

      // CICS commarea
      byte[] Commarea = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nasmj002 - ECI Asynchronous Calls with Message Qualifiers");
      System.out.println("       -j     JGate                           -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port                      -> "+theseParms.sPort);
      System.out.println("       -list  list systems only               -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server                     -> "+theseParms.sServer);
      System.out.println("       -a     application                     -> "+theseParms.sProgname);
      System.out.println("       -wait  eci_get_specific_reply_wait     -> "+theseParms.bWait);
      System.out.println("       -2sec  delay between async calls       -> "+theseParms.bAsyncDelay);
      System.out.println("       -clear clear outstanding replies first -> "+theseParms.bClearReplies);
      System.out.println("       -i     # iterations                    -> "+theseParms.sIterations);
      System.out.println("       -u     userid                          -> "+theseParms.sUserid);
      System.out.println("       -p     password                        -> "+theseParms.sPassword);
      System.out.println("       -d     delay                           -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x     exit on ECI error               -> "+theseParms.bExitNow);
      System.out.println("       -!     debug                           -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java asmj002 <parameters>");
         System.out.println(  "  e.g. java asmj002 -s AlsKix -wait");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("asmj002");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ------------------------------------------------------------------
      // If -clear option specified clear outstanding replies.
      // ------------------------------------------------------------------

      if (theseParms.bClearReplies)
      {
         System.out.println("\nRequest to clear outstanding replies");

         boolean bMoreReplies = true;
         while (bMoreReplies)
         {
            ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_GET_REPLY,
                                                     theseParms.sServer,
                                                     theseParms.sUserid,
                                                     theseParms.sPassword,
                                                     theseParms.sProgname,
                                                     theseParms.sTransid,
                                                     Commarea);
            // Flow get_reply request
            try
            {
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e)
            {
               T.ln(null,"ERROR: Caught Exception: ");
               T.ex(null,e);
               System.out.println("\n--- Error during eci_get_reply flow ---\n");
               System.out.println("--- Caught Exception: " + e);
               // ----------------------------------------------------------------------
               // Call class closeJGate to close Java Gateway connection and exit
               // ----------------------------------------------------------------------
               setJGate.closeJGate(jgaConnection);
               System.exit(0);
            }
            switch(eciRequest.Cics_Rc)
            {
               case ECIRequest.ECI_NO_ERROR :
                  System.out.println("-> Cleared Message_Qualifier "+eciRequest.Message_Qualifier);
                  break;
               case ECIRequest.ECI_ERR_NO_REPLY :
                  System.out.println("No more outstanding replies");
                  bMoreReplies = false;
                  break;
               case ECIRequest.ECI_ERR_TRANSACTION_ABEND :
                  if (eciRequest.Abend_Code.equals("AEY7"))
                  {
                     System.out.println("Ignoring security error");
                     break;
                  }
               // if not AEY7 or other codes above - finish.
               default :
                  System.out.println("--- Unexpected return code on eci_get_reply: "+
                              eciRequest.Cics_Rc);
                  // ----------------------------------------------------------------------
                  // Call class closeJGate to close Java Gateway connection and exit
                  // ----------------------------------------------------------------------
                  setJGate.closeJGate(jgaConnection);
                  System.exit(0);
            } // end-switch
         } // end-while
      } // end-if

      // ------------------------------------------------------------------
      // Full Contructor ECIRequest() required for asynchronous calls
      // ------------------------------------------------------------------

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // output banner every iteration
         System.out.println("\n#  Call + Message_Qualifier       Commarea-Data      Length CICS Rc  Abend");
         System.out.println("-- ------------------------------ -----------------  ------ -------  -----");

         // issue 5 async calls with message_qualifiers 5 to 1
         iMsg_Qual = 5;
         for (iEci_Count=1; iEci_Count<=5; iEci_Count++)
         {
            ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_ASYNC,
                                                     theseParms.sServer,
                                                     theseParms.sUserid,
                                                     theseParms.sPassword,
                                                     theseParms.sProgname,
                                                     theseParms.sTransid,
                                                     Commarea,
                                                     0,
                                                     ECIRequest.ECI_NO_EXTEND,
                                                     ECIRequest.ECI_LUW_NEW,
                                                     iMsg_Qual,
                                                     null );

            // set commarea to dashes
            for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
               eciRequest.Commarea[iCounter]=45;

            // Convert ASCII bytes to ASCII characters
            String sDashData  = new String(eciRequest.Commarea);

            // Flow asynchronous request
            try
            {
               iCallsIssued++;
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e)
            {
               T.ln(null,"ERROR: Caught Exception: ");
               T.ex(null,e);
               System.out.println("\n--- Error during eci_async flow ---\n");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }

            System.out.println(iIt_Count+"  Asynchronous call + MsgQual " + iMsg_Qual + "  " +
                        sDashData.substring(0,eciRequest.Commarea_Length-1) +
                        "  " + eciRequest.Commarea_Length +"     "+
                        eciRequest.Cics_Rc + "        "+eciRequest.Abend_Code );

            // for debug purposes
            T.ln(null,"Back again : AbendCode = " + eciRequest.Abend_Code);
            T.ln(null,"eci RC " + eciRequest.Cics_Rc);

            // Check result
            if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
            {
               iReplyOK++;
            }
            else
            {
               if (theseParms.bExitNow)
               {
                  System.out.println("\n--- ECI CICS_RC error - aborting program");
                  break iteration_loop;
               }
            }

            // if async call fails do not carry on ..
            if (eciRequest.Cics_Rc != 0)
            {
               System.out.println("\n--- Unexpected return code: "+eciRequest.Cics_Rc);
               System.out.println("\n--- Since asynchronous call failed, program is terminating.");
               break iteration_loop;
            } // end-async-error

            // If -2sec option specified delay next async call for 2 seconds.
            // This allows the user to see the different timestamps
            // on the get_reply responses, and ensure they are in reverse
            // order.
            if (theseParms.bAsyncDelay)
               try
               {
                  Thread.sleep(2000);
               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error during sleep() ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            // set Message_Qualifier to one less than previous
            iMsg_Qual--;
         } // end-iEci_Count loop to issue async calls

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT or ECI_GET_REPLY (loop up to 9 seconds)
         // ------------------------------------------------------------------

         // ------------------------------------------------------------------
         // Use Basic Constructor ECIRequest() for REPLY
         // ------------------------------------------------------------------
         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_GET_SPECIFIC_REPLY_WAIT,
                                                  theseParms.sServer,
                                                  theseParms.sUserid,
                                                  theseParms.sPassword,
                                                  theseParms.sProgname,
                                                  theseParms.sTransid,
                                                  Commarea);

         // if -wait option specified as input parameter issue eci_get_reply_wait
         if (theseParms.bWait)
         {
            eciRequest.Call_Type = ECIRequest.ECI_GET_SPECIFIC_REPLY_WAIT;

            iMsg_Qual = 1;
            for (iEci_Count=1; iEci_Count<=5; iEci_Count++)
            {
               eciRequest.Message_Qualifier = iMsg_Qual;
               sGetReply = "eci_get_specific_reply_wait "+iMsg_Qual;

               // flow ECI request
               try
               {
                  iCallsIssued++;
                  jgaConnection.flow(eciRequest);
               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error during eci_specific_reply_wait flow ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }

               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea);

               // Display result
               System.out.println(iIt_Count+"  "+sGetReply+"  "+
                           ComData.substring(0,eciRequest.Commarea_Length-1) +
                           "  " + eciRequest.Commarea_Length +"     "+
                           eciRequest.Cics_Rc + "        " +
                           eciRequest.Abend_Code );

               // increment Message_Qualifier for next eci call
               iMsg_Qual++;

               // Check result
               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
               {
                  iReplyOK++;
               }
               else
               {
                  if (theseParms.bExitNow)
                  {
                     System.out.println("\n--- ECI CICS_RC error - aborting program");
                     break iteration_loop;
                  }
               }
            } // end-iEci_Count
         }  // end-if-bWait
         else
         {
            // -wait option not specified so issue enter
            //           eci_get_specific_reply loop
            iMsg_Qual = 1;
            for (iEci_Count=1; iEci_Count<=5; iEci_Count++)
            {
               bReply_Rcvd = false;
               eciRequest.Message_Qualifier = iMsg_Qual;
               eciRequest.Call_Type = ECIRequest.ECI_GET_SPECIFIC_REPLY;
               iCallsIssued++; // only want to increment this total once
               for (iReply_Count=1; (iReply_Count<=9 && !bReply_Rcvd);
                    iReply_Count++)
               {
                  // flow ECI request
                  try
                  {
                     jgaConnection.flow(eciRequest);
                  }
                  catch (Exception e)
                  {
                     T.ln(null,"ERROR: Caught Exception: ");
                     T.ex(null,e);
                     System.out.println("\n--- Error during eci_specific_reply flow ---\n");
                     System.out.println("--- Caught Exception: " + e);
                     break iteration_loop;
                  }
                  sGetReply = "eci_get_specific_reply#"+iReply_Count+" MQ "+iMsg_Qual;
                  // check return code ...
                  switch(eciRequest.Cics_Rc)
                  {
                     case ECIRequest.ECI_NO_ERROR :
                        bReply_Rcvd = true;
                        break;
                     case ECIRequest.ECI_ERR_NO_REPLY :
                        // Delay next ECI call for one second (up to 9 secs)
                        try
                        {
                           Thread.sleep(1000);
                        }
                        catch (Exception e)
                        {
                           T.ln(null,"ERROR: Caught Exception: ");
                           T.ex(null,e);
                           System.out.println("\n--- Error during sleep() ---\n");
                           System.out.println("--- Caught Exception: " + e);
                           break iteration_loop;
                        }
                        break;
                     default :
                        System.out.println(iIt_Count+"  "+sGetReply+"  "+
                                    "Unexpected return code:   " +
                                    eciRequest.Cics_Rc + "        " +
                                    eciRequest.Abend_Code );
                        break iteration_loop;
                  } // end-switch
               } // end-for-iReply_Count loop
               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea);
               // Display result
               System.out.println(iIt_Count+"  "+sGetReply+"  "+
                           ComData.substring(0,eciRequest.Commarea_Length-1) +
                           "  " + eciRequest.Commarea_Length +"     "+
                           eciRequest.Cics_Rc + "        " +
                           eciRequest.Abend_Code );

               // increment Message_Qualifier for next eci call
               iMsg_Qual++;

               // Check result
               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
               {
                  iReplyOK++;
               }
               else
               {
                  if (theseParms.bExitNow)
                  {
                     System.out.println("\n--- ECI CICS_RC error - aborting program");
                     break iteration_loop;
                  }
               }
            } // end-for-iEci_Count loop
         } // end-else

         // Delay next ECI call if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("asmj002", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
