//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      asyj002 - Asynchronous calls with
//                ECI_GET_REPLY_WAIT or ECI_GET_REPLY loop
//p
//      asyj002 is a Java program that on each requested iteration
//      issues an asynchronous call (without callback or message qualifier).
//      This is followed either by an ECI_GET_REPLY_WAIT (if the -wait option
//      is specified as an override), or an ECI_GET_REPLY (the default).
//      In the latter case up to nine ECI_GET_REPLY calls are issued to
//      obtain the response from the server. Between each attempt to
//      receive this reply there is a one second delay to allow for network
//      traffic or a busy server.
//p
//      With ECI_GET_REPLY calls the output shows which call obtained the
//      response. For example, "eci_get_reply #3" means that the response was
//      obtained on the third attempt. If the result shows "eci_get_reply #9"
//      with a CICS Rc of -5, this indicates that the response was not
//      received after nine attempts. After all iterations have run, a banner
//      displays the number of successful iterations where asynchronous calls
//      returned a positive reply.
//p
//      Prior to each asynchronous call the commarea is initialised to
//      18 dashes. The server returns the current date and timestamp.
//      The default CICS server program is the supplied SRVTIME
//      which is compatible on all CICS servers. If desired, a different server
//      program can be supplied as an input parameter to this application.
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Parameters can be overridden via command line.
//p
//      Issue "asyj002 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.asyj002 -i 3
//p
//      The sample input data implies a default of ECI_GET_REPLY. The sample
//      output data shows that a response to each ECI_SYNC call was received
//      on the second attempt.
//p
//      Sample Output
//x
//      asyj002 - ECI Asynchronous Call Testing
//             -j     JGate              -> hollings
//             -r     JGate port         -> 2006
//             -list  list systems only  -> false
//             -s     CICS server        -> (not specified)
//             -a     application        -> SRVTIME
//             -wait  eci_get_reply_wait -> false
//             -i     # iterations       -> 3
//             -u     userid             -> SYSAD
//             -p     password           -> SYSAD
//             -d     delay              -> 0 seconds
//             -!     debug              -> false
//             -?     Display settings only
//
//      --- asyj002 started 12/06/98 07:06
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      #  Call               Commarea-Data      Length CICS Rc  Abend
//      -- ------------------ -----------------  ------ -------  -----
//      1  Async call ->      -----------------  18     0
//      1  eci_get_reply #2   12/06/98 08:06:34  18     0
//      2  Async call ->      -----------------  18     0
//      2  eci_get_reply #2   12/06/98 08:06:35  18     0
//      3  Async call ->      -----------------  18     0
//      3  eci_get_reply #2   12/06/98 08:06:36  18     0
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- asyj002 completed 12/06/98 07:06
//
//      --- Calls successful ... 6 out of 6
//      --- Iterations run ..... 3 out of 3
//
//p
//      History log:
//u
//l     Written : Apr 97 - Alan Hollingshead
//l     Updated : Apr 98 - uses common classes
//
//
package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;


public class asyj002
{
   public static void main(String[] args)
   {
      // set up variables
      int     iCounter     = 0;
      int     iIt_Count    = 0;
      int     iReply_Count = 0;
      int     iReplyOK     = 0;
      int     iCallsIssued = 0;
      String  sDebug       = " ";
      String  sWait        = " ";
      String  sGetReply    = "";
      boolean bReply_Rcvd  = false;

      // CICS commarea
      byte[] Commarea      = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nasyj002 - ECI Asynchronous Call Testing");
      System.out.println("       -j     JGate              -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port         -> "+theseParms.sPort);
      System.out.println("       -list  list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server        -> "+theseParms.sServer);
      System.out.println("       -a     application        -> "+theseParms.sProgname);
      System.out.println("       -wait  eci_get_reply_wait -> "+theseParms.bWait);
      System.out.println("       -i     # iterations       -> "+theseParms.sIterations);
      System.out.println("       -u     userid             -> "+theseParms.sUserid);
      System.out.println("       -p     password           -> "+theseParms.sPassword);
      System.out.println("       -d     delay              -> "+theseParms.sDelay+" seconds");
      System.out.println("       -!     debug              -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java asyj002 <parameters>");
         System.out.println(  "  e.g. java asyj002 -s AlsKix -wait");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("asyj002");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }


      // ----------------------------------------------------------------------
      // Full Contructor ECIRequest() required for asynchronous calls
      // ----------------------------------------------------------------------

      System.out.println("\n#  Call               Commarea-Data      Length CICS Rc  Abend");
      System.out.println("-- ------------------ -----------------  ------ -------  -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // output banner every 10 iterations
         if (iIt_Count%10==0)
         {
            System.out.println("\n#   Call               Commarea-Data      Length CICS Rc  Abend");
            System.out.println("--  ------------------ -----------------  ------ -------  -----");
         } // end-if

         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_ASYNC,
                                                  theseParms.sServer,
                                                  theseParms.sUserid,
                                                  theseParms.sPassword,
                                                  theseParms.sProgname,
                                                  theseParms.sTransid,
                                                  Commarea,
                                                  0,
                                                  ECIRequest.ECI_NO_EXTEND,
                                                  ECIRequest.ECI_LUW_NEW,
                                                  0,
                                                  null );

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         String sDashData  = new String(eciRequest.Commarea,0);

         // Flow asynchronous request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_async flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
            iReplyOK++;

         String sSpace7  = "       ";
         String sGap1 = sSpace7.substring(1,(8-(String.valueOf(eciRequest.Cics_Rc).length())));

         System.out.println(iIt_Count+"  Async call ->      " +
                     sDashData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc +"  "+ sGap1+
                     eciRequest.Abend_Code );

         // for debug purposes
         T.ln(null,"Back again : AbendCode = " + eciRequest.Abend_Code);
         T.ln(null,"eci RC " + eciRequest.Cics_Rc);

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT or ECI_GET_REPLY (loop up to 9 seconds)
         // ------------------------------------------------------------------

         // if -wait option specified as input parameter issue eci_get_reply_wait

         iCallsIssued++;

         if (theseParms.bWait)
         {
            eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;
            sGetReply = "eci_get_reply_wait ";

            // flow ECI request
            try
            {
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e)
            {
               T.ln(null,"ERROR: Caught Exception: ");
               T.ex(null,e);
               System.out.println("\n--- Error during eci_get_reply_wait flow ---\n");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }
         else
         {
            // -wait option not specified so issue enter eci_get_reply loop
            bReply_Rcvd = false;
            for (iReply_Count=1; (iReply_Count<=9 && !bReply_Rcvd);
                 iReply_Count++)
            {
               eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY;

               // flow ECI request
               try
               {
                  jgaConnection.flow(eciRequest);
               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error during eci_get_reply flow ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
               sGetReply = "eci_get_reply #"+iReply_Count+"   ";
               // check for good return code ...
               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
               {
                  bReply_Rcvd = true;
               }
               // else delay for one second and try again (up to 9 times)
               else
               {
                  // Delay next ECI call for one second
                  try
                  {
                     Thread.sleep(1000);
                  }
                  catch (Exception e)
                  {
                     T.ln(null,"ERROR: Caught Exception: ");
                     T.ex(null,e);
                     System.out.println("\n--- Error during sleep() ---\n");
                     System.out.println("--- Caught Exception: " + e);
                     break iteration_loop;
                  }
               } // end-else
            } // end-for loop
         } // end-else

         // Convert ASCII bytes to ASCII characters
         String ComData  = new String(eciRequest.Commarea,0);

         sSpace7  = "       ";
         sGap1 = sSpace7.substring(1,(8-(String.valueOf(eciRequest.Cics_Rc).length())));

         // Display result
         System.out.println(iIt_Count+"  "+sGetReply+
                  ComData.substring(0,eciRequest.Commarea_Length-1) +
                  "  " + eciRequest.Commarea_Length +"     "+
                  eciRequest.Cics_Rc +"  "+ sGap1+
                  eciRequest.Abend_Code );

         // keep tally of successful return of commarea
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
         {
            iReplyOK++;
         } // end-if
         // Delay next ECI call if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("asyj002", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
