//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      ecij123 - Three simple calls to test the CICS Gateway for Java
//p
//      The following calls are tested:
//u
//l               - eci_sync
//l               - eci_async (no callback)
//l               - eci_get_reply_wait
//p
//      This application issues three simple ECI calls that can be iterated
//      ad nauseum. The first is an ECI_SYNC call. This is followed by an
//      ECI_ASYNC call (no callback) to which an ECI_GET_REPLY_WAIT awaits
//      the response.
//p
//      Prior to each ECI call the commarea is initialised to
//      18 dashes. The server returns the current date and timestamp for
//      the ECI_SYNC and ECI_GET_REPLY_WAIT calls.
//      The default CICS server program is the supplied SRVTIME
//      which is compatible on all CICS servers. If desired, a different server
//      program can be supplied as an input parameter to this application.
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Parameters can be overridden via command line.
//p
//      Issue "ecij123 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.ecij123 -j local: -i 2
//p
//      Sample Output
//x
//      ecij123 - Three Simple ECI Calls
//             -j     JGate                    -> local:
//             -r     JGate port               -> 2006
//             -list  list systems only        -> false
//             -s     CICS server              -> (not specified)
//             -a     application              -> SRVTIME
//             -i     # iterations             -> 2
//             -u     userid                   -> SYSAD
//             -p     password                 -> SYSAD
//             -d     delay between iterations -> 0 seconds
//             -x     exit on ECI error        -> false
//             -!     debug                    -> false
//             -?     Display settings only
//
//      --- ecij123 started 12/06/98 12:08
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      #  Call               Commarea-Data      Length CICS Rc  Abend
//      -- ------------------ -----------------  ------ -------  -----
//      1  eci_sync           12/06/98 13:08:20  18     0        null
//      1  eci_async          -----------------  18     0        null
//      1  eci_get_reply_wait 12/06/98 13:08:21  18     0        null
//
//      2  eci_sync           12/06/98 13:08:21  18     0        null
//      2  eci_async          -----------------  18     0        null
//      2  eci_get_reply_wait 12/06/98 13:08:21  18     0        null
//
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- ecij123 completed 12/06/98 12:08
//
//      --- Calls successful ... 6 out of 6
//      --- Iterations run ..... 2 out of 2
//p
//      History log:
//u
//l     Written : Apr 97 - Alan Hollingshead
//l     Updated : Apr 98 - uses common classes
//

package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class ecij123
{
   public static void main(String[] args)
   {
      int     iEci_Call      = 0;
      int     iLUW           = 0;
      int     iIt_Count      = 0;
      int     iCounter       = 0;
      int     iCallsIssued   = 0;
      int     iReply_Count   = 0;
      int     iReplyOK       = 0;
      String  sGetReply      = "";
      boolean bReply_Rcvd    = false;

      // CICS commarea
      byte[] Commarea = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\necij123 - Three Simple ECI Calls") ;
      System.out.println("       -j     JGate                    -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port               -> "+theseParms.sPort);
      System.out.println("       -list  list systems only        -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server              -> "+theseParms.sServer);
      System.out.println("       -a     application              -> "+theseParms.sProgname);
      System.out.println("       -i     # iterations             -> "+theseParms.sIterations);
      System.out.println("       -u     userid                   -> "+theseParms.sUserid);
      System.out.println("       -p     password                 -> "+theseParms.sPassword);
      System.out.println("       -d     delay between iterations -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x     exit on ECI error        -> "+theseParms.bExitNow);
      System.out.println("       -!     debug                    -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java ecij123 <parameters>");
         System.out.println(  "  e.g. java ecij123 -s AlsKix -i 5");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("ecij123");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ------------------------------------------------------------------
      // Use extended contructor ECIRequest() for synchronous call
      // ------------------------------------------------------------------

      System.out.println("\n#  Call               Commarea-Data      Length CICS Rc  Abend");
      System.out.println("-- ------------------ -----------------  ------ -------  -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // output banner every 10 iterations
         if (iIt_Count%10==0)
         {
            System.out.println("\n#   Call               Commarea-Data      Length CICS Rc  Abend");
            System.out.println("--  ------------------ -----------------  ------ -------  -----");
         }

         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                                  theseParms.sServer,
                                                  theseParms.sUserid,
                                                  theseParms.sPassword,
                                                  theseParms.sProgname,
                                                  theseParms.sTransid,
                                                  Commarea,
                                                  0,
                                                  ECIRequest.ECI_NO_EXTEND,
                                                  ECIRequest.ECI_LUW_NEW );

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         String sDashData  = new String(eciRequest.Commarea);

         // Flow synchronous request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_sync flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         // ,0 required for OS/390
         String ComData  = new String(eciRequest.Commarea,0);

         System.out.println(iIt_Count+"  eci_sync           " +
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // ------------------------------------------------------------------
         // ECI_ASYNC
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_ASYNC;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea);

         // flow ECI request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_async flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea);

         // Display result
         System.out.println(iIt_Count+"  eci_async          "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         sDashData = new String(eciRequest.Commarea);

         // flow ECI request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_get_reply_wait flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea);

         // Display result
         System.out.println(iIt_Count+"  eci_get_reply_wait "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code + "\n");

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // Delay next ECI call if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("ecij123", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
