//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      ecijtpn - ECI_SYNC_TPN and ECI_ASYNC_TPN Calls
//p
//      This application issues the following ECI calls:
//o
//l             eci_sync_tpn
//l             eci_async_tpn
//l             eci_get_reply_wait
//l             eci_sync
//l             eci_async
//l             eci_get_reply_wait
//p
//      The program also uses the method isTPNTransid to confirm
//      whether tpn is being used.
//p
//      Note: tpn is currently not supported on Unix platforms.
//p
//      Testing should include:
//o
//l     Issuing the calls using the default transid CPMI
//l     Issuing the calls specifying your own transid. You must
//        define your own transid in the server. For example, on
//        CICS for OS/2  CPMI runs program FAAMIR; therefore you
//        can define your own transid, eg MYMI, to call program FAAMIR.
//        The example below shows how to override CPMI with MYMI.
//        Note that the value of transid is case-sensitive.
//l     Testing with a mirror less than 4-characters in length.
//l     With a transid undefined on the server. The _tpn calls should
//        abend, but the following non-tpn calls should use the default
//        CPMI transid and work successfully.
//p
//      Prior to each ECI call the commarea is initialised to
//      18 dashes. The default CICS server program is the supplied SRVTIME
//      which is compatible on all CICS servers. If desired, a different server
//      program can be supplied as an input parameter to this application.
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Parameters can be overridden via command line.
//p
//      Issue "ecijtpn -?" for help and default settings.
//p
//      Note: the minimum number of iterations is 2
//p
//      Sample Input
//x
//      java jgtest.eci.ecijtpn -j local: -tran MYMI
//p
//      Sample Output
//x
//      ecijtpn - ECI_SYNC_TPN and ECI_ASYNC_TPN Calls
//             -j     JGate                    -> local:
//             -r     JGate port               -> 2006
//             -list  list systems only        -> false
//             -s     CICS server              -> (not specified)
//             -a     application              -> SRVTIME
//             -i     # iterations (minimum 2) -> 2
//             -u     userid                   -> SYSAD
//             -p     password                 -> SYSAD
//             -d     delay between iterations -> 0 seconds
//             -x     exit on ECI error        -> false
//             -!     debug                    -> false
//             -?     Display settings only
//
//      --- ecijtpn started 12/06/98 12:14
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      #  Call                Mirror  Commarea Data      Length CICS Rc  Abend
//      -- ------------------  ------  -----------------  ------ -------  -----
//      1  eci_sync_tpn        MYMI    12/06/98 13:14:32  18     0        null
//         isTPNTransid  -->   true
//      1  eci_async_tpn       MYMI    -----------------  18     0        null
//         isTPNTransid  -->   true
//      1  eci_get_reply_wait  MYMI    12/06/98 13:14:32  18     0        null
//
//      2  eci_sync            MYMI    12/06/98 13:14:32  18     0        null
//         isTPNTransid  -->   false
//      2  eci_async           MYMI    -----------------  18     0        null
//         isTPNTransid  -->   false
//      2  eci_get_reply_wait  MYMI    12/06/98 13:14:33  18     0        null
//
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- ecijtpn completed 12/06/98 12:14
//
//      --- Calls successful ... 6 out of 6
//      --- Iterations run ..... 2 out of 2
//
//p
//      History log:
//u
//l     Written : Apr 97 - Alan Hollingshead
//l     Updated : Apr 98 - uses common classes
//
package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class ecijtpn
{
   public static void main(String[] args)
   {
      int     iEci_Call      = 0;
      int     iLUW           = 0;
      int     iIt_Count      = 0;
      int     iCounter       = 0;
      int     iCallsIssued   = 0;
      int     iReply_Count   = 0;
      int     iReplyOK       = 0;
      String  sGetReply      = "";
      String  sSyncCall      = "";
      String  sAsyncCall     = "";
      boolean bReply_Rcvd    = false;

      // CICS commarea
      byte[] Commarea = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // set minimum iterations to 2 to test tpn and non-tpn calls
      if (theseParms.iIterations < 2)
      {
         theseParms.iIterations = 2;
         theseParms.sIterations = "2";
      }

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\necijtpn - ECI_SYNC_TPN and ECI_ASYNC_TPN Calls");
      System.out.println("       -j     JGate                    -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port               -> "+theseParms.sPort);
      System.out.println("       -list  list systems only        -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server              -> "+theseParms.sServer);
      System.out.println("       -a     application              -> "+theseParms.sProgname);
      System.out.println("       -i     # iterations (minimum 2) -> "+theseParms.sIterations);
      System.out.println("       -u     userid                   -> "+theseParms.sUserid);
      System.out.println("       -p     password                 -> "+theseParms.sPassword);
      System.out.println("       -d     delay between iterations -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x     exit on ECI error        -> "+theseParms.bExitNow);
      System.out.println("       -!     debug                    -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java ecijtpn <parameters>");
         System.out.println(  "  e.g. java ecijtpn -s AlsKix -i 5");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("ecijtpn");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ------------------------------------------------------------------
      // Use extended contructor ECIRequest() for synchronous call
      // ------------------------------------------------------------------

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC_TPN,
                                               theseParms.sServer,
                                               theseParms.sUserid,
                                               theseParms.sPassword,
                                               theseParms.sProgname,
                                               theseParms.sTransid,
                                               Commarea,
                                               0,
                                               ECIRequest.ECI_NO_EXTEND,
                                               ECIRequest.ECI_LUW_NEW );

      System.out.println("\n#  Call                Mirror  Commarea Data      Length CICS Rc  Abend");
      System.out.println("-- ------------------  ------  -----------------  ------ -------  -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // output banner every 10 iterations
         if (iIt_Count%10==0)
         {
            System.out.println("\n#  Call                Mirror  Commarea Data      Length CICS Rc  Abend");
            System.out.println("-- ------------------  ------  -----------------  ------ -------  -----");
         }

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         String sDashData  = new String(eciRequest.Commarea);

         // Flow synchronous request
         if (iIt_Count%2==0)
         {
            eciRequest.Call_Type = ECIRequest.ECI_SYNC;
            sSyncCall = "eci_sync     ";
         }
         else
         {
            eciRequest.Call_Type = ECIRequest.ECI_SYNC_TPN;
            sSyncCall = "eci_sync_tpn ";
         }
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_sync_tpn flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         // ,0 required for OS/390
         String ComData  = new String(eciRequest.Commarea,0);

         System.out.println(iIt_Count + "  " + sSyncCall + "       " +
                     theseParms.sTransid + "    " +
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // check isTPNTransid method
         System.out.println("   isTPNTransid  -->   "
                             + eciRequest.isTPNTransid());

         if ((eciRequest.isTPNTransid() == false) && (iIt_Count%2!=0) )
         {
            System.out.println("\n--- isTPNTransid error - expected 'true' - aborting program");
            break iteration_loop;
         }

         if ((eciRequest.isTPNTransid() == true) && (iIt_Count%2==0) )
         {
            System.out.println("\n--- isTPNTransid error - expected 'false' - aborting program");
            break iteration_loop;
         }

         // ------------------------------------------------------------------
         // ECI_ASYNC
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_ASYNC_TPN;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea);

         if (iIt_Count%2==0)
         {
            eciRequest.Call_Type = ECIRequest.ECI_ASYNC;
            sAsyncCall = "eci_async    ";
         }
         else
         {
            eciRequest.Call_Type = ECIRequest.ECI_ASYNC_TPN;
            sAsyncCall = "eci_async_tpn";
         }
         // flow ECI request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_async flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea);

         // Display result
         System.out.println(iIt_Count + "  " + sAsyncCall + "       " +
                     theseParms.sTransid + "    " +
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // check isTPNTransid method
         System.out.println("   isTPNTransid  -->   "
                             + eciRequest.isTPNTransid());

         if ((eciRequest.isTPNTransid() == false) && (iIt_Count%2!=0) )
         {
            System.out.println("\n--- isTPNTransid error - expected 'true' - aborting program");
            break iteration_loop;
         }

         if ((eciRequest.isTPNTransid() == true) && (iIt_Count%2==0) )
         {
            System.out.println("\n--- isTPNTransid error - expected 'false' - aborting program");
            break iteration_loop;
         }

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         sDashData = new String(eciRequest.Commarea);

         // flow ECI request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_get_reply_wait flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea);

         // Display result
         System.out.println(iIt_Count+"  eci_get_reply_wait  "+
                     theseParms.sTransid + "    " +
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code + "\n");

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // Delay next ECI call if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("ecijtpn", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
