//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      synj001 - Extended ECI Synchronous Calls with multiple LUWs
//p
//      synj001 is a Java application that on each requested iteration
//      issues sets of six ECI synchronous calls for each LUW. The first
//      two calls are followed by an ECI_BACKOUT. The fourth and fifth
//      calls are followed by an ECI_COMMIT. The commarea on each call
//      is initialised to 18 dashes. The server returns the current date
//      and timestamp. The default CICS server program is the supplied SRVTIME
//      which is compatible on all CICS servers. If desired, a different server
//      program can be supplied as an input parameter to this application.
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Parameters can be overridden via command line.
//p
//      Issue "synj001 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.synj001 -j local: -s TCP-TCLR
//p
//      Sample Output
//x
//      synj001 - Extended ECI Synchronous Calls
//             -j     JGate              -> local:
//             -r     JGate port         -> 2006
//             -list  list systems only  -> false
//             -s     CICS server        -> TCP-TCLR
//             -a     application        -> SRVTIME
//             -luw   # concurrent luws  -> 1
//             -i     # iterations       -> 1
//             -u     userid             -> SYSAD
//             -p     password           -> SYSAD
//             -d     delay              -> 0 seconds
//             -x     exit on ECI error  -> false
//             -!     debug              -> false
//             -?     Display settings only
//
//      --- synj001 started 5/29/98 5:12 PM
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Iteration: 1 of 1
//
//      # LUW Extend-Mode  Commarea-Data      Length CICS Rc  Abend  LUW Token
//      - --- -----------  -------------      ------ -------  -----  ---------
//      1  1  ECI_EXTENDED 29/05/98 17:12:55  18     0        null   2
//      2  1  ECI_EXTENDED 29/05/98 17:12:55  18     0        null   2
//      3  1  ECI_BACKOUT  -----------------  18     0        null   0
//      4  1  ECI_EXTENDED 29/05/98 17:12:56  18     0        null   2
//      5  1  ECI_EXTENDED 29/05/98 17:12:56  18     0        null   2
//      6  1  ECI_COMMIT   -----------------  18     0        null   0
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- synj001 completed 5/29/98 5:12 PM
//
//      --- Calls successful ... 6 out of 6
//      --- Iterations run ..... 1 out of 1
//p
//      History log:
//u
//l     Written : Nov 96 - Alan Hollingshead
//l     Updated : Apr 98 - new format using common classes
//

package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class synj001
{
   public static void main(String[] args)
   {
      int iEci_Call      = 0;
      int iLUW           = 0;
      int iIt_Count      = 0;
      int iCallsIssued   = 0;
      int iReplyOK       = 0;
      String sExtMode    = "ECI_BACKOUT";

      //  CICS commarea
      byte[] Commarea = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nsynj001 - Extended ECI Synchronous Calls") ;
      System.out.println("       -j     JGate              -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port         -> "+theseParms.sPort);
      System.out.println("       -list  list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server        -> "+theseParms.sServer);
      System.out.println("       -a     application        -> "+theseParms.sProgname);
      System.out.println("       -luw   # concurrent luws  -> "+theseParms.sNumLUWs);
      System.out.println("       -i     # iterations       -> "+theseParms.sIterations);
      System.out.println("       -u     userid             -> "+theseParms.sUserid);
      System.out.println("       -p     password           -> "+theseParms.sPassword);
      System.out.println("       -d     delay              -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x     exit on ECI error  -> "+theseParms.bExitNow);
      System.out.println("       -!     debug              -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java synj001 <parameters>");
         System.out.println(  "  e.g. java synj001 -s AlsKix -i 5");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("synj001");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ----------------------------------------------------------------------
      // Extended Contructor ECIRequest()
      // ----------------------------------------------------------------------

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                               theseParms.sServer,
                                               theseParms.sUserid,
                                               theseParms.sPassword,
                                               theseParms.sProgname,
                                               theseParms.sTransid,
                                               Commarea,
                                               0,
                                               0,
                                               0);

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // display header
         System.out.println("\n--- Iteration: "+iIt_Count+" of "+theseParms.iIterations+"\n");
         System.out.println("# LUW Extend-Mode  Commarea-Data      Length CICS Rc  Abend  LUW Token");
         System.out.println("- --- -----------  -------------      ------ -------  -----  ---------");

         // array for LUW tokens
         int[] aiTokens = new int[theseParms.iNumLUWs+1];
         for (iLUW=1; iLUW<=theseParms.iNumLUWs; iLUW++)
         // initialise Tokens to zero
            aiTokens[iLUW]=0;

         for (iEci_Call=1; iEci_Call<=6; iEci_Call++)
         {
            switch(iEci_Call)
            {
               case 3  : eciRequest.Extend_Mode=ECIRequest.ECI_BACKOUT;
                         sExtMode = "ECI_BACKOUT ";
                         break;
               case 6  : eciRequest.Extend_Mode=ECIRequest.ECI_COMMIT;
                         sExtMode = "ECI_COMMIT  ";
                         break;
               default : eciRequest.Extend_Mode=ECIRequest.ECI_EXTENDED;
                         sExtMode = "ECI_EXTENDED";
                         break;
            } // end-switch

            // for each LUW
            for (iLUW=1; iLUW<=theseParms.iNumLUWs; iLUW++) try
            {
               // set commarea to dashes
               for (int iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
               {
                  // reset commarea to dashes
                  eciRequest.Commarea[iCounter]=45;
               }
               // use correct LUW token
               eciRequest.Luw_Token=aiTokens[iLUW];

               // flow ECI request
               iCallsIssued++;
               jgaConnection.flow(eciRequest);

               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea,0);

               String sSpace7  = "       ";
               String sGap1 = sSpace7.substring(1,(8-(String.valueOf(eciRequest.Cics_Rc).length())));

               // Display result
               System.out.println(iEci_Call+"  "+iLUW+"  "+
                    sExtMode+" "+
                    ComData.substring(0,eciRequest.Commarea_Length-1)+"  "+
                    eciRequest.Commarea_Length +"     "+
                    eciRequest.Cics_Rc +"  "+ sGap1 +
                    eciRequest.Abend_Code +"   "+
                    eciRequest.Luw_Token);

               // Check result for ECI_EXTENDED calls
               if ((eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) &&
                   (sExtMode.equals("ECI_EXTENDED")) &&
                   (!ComData.equals("------------------"))  )
               {
                  iReplyOK++;
               }
               else
               {
                  if (theseParms.bExitNow)
                  {
                     System.out.println("\n--- ECI CICS_RC error - aborting program");
                     break iteration_loop;
                  }
               }

               // Check result for non-ECI_EXTENDED calls
               if ((eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) &&
                   (!sExtMode.equals("ECI_EXTENDED")) )
               {
                  iReplyOK++;
               }
               else
               {
                  if (theseParms.bExitNow)
                  {
                     System.out.println("\n--- ECI CICS_RC error - aborting program");
                     break iteration_loop;
                  }
               }

               // Assign new LUW token
               aiTokens[iLUW]=eciRequest.Luw_Token;

               // Delay next ECI call if delay parameter specified
               try
               {
                  Thread.sleep(theseParms.iDelay);
               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error during sleep() ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            }  //end LUW for-loop
            catch (Exception e)
            {
               T.ln(null,"ERROR: Caught Exception: ");
               T.ex(null,e);
               System.out.println("\n--- Error returned from JGate flow ---\n");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }  // end iEci_Call for-loop
      }  // end iIt_Count for-loop

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("synj001", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
