//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      synj0c3 - Detect Maximum Commarea Length
//p
//      synj0c3 issues non-extended ECI synchronous calls incrementing
//      the commarea length until the maximum length (governed
//      by CICS server and/or communication protocol) has been reached.
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Issue "synj0c3 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.synj0c3 -cmlen 32700 -+
//p
//      Sample Output
//x
//      synj0c3 - Detect Maximum Commarea Length
//             -j     JGate                      -> hollings
//             -r     JGate port                 -> 2006
//             -list  list systems only          -> false
//             -s     CICS server                -> (not specified)
//             -cmlen initial commarea length    -> 32700
//             -+     increment commarea length  -> true
//             -i     # iterations               -> 1
//             -a     application                -> SRVTIME
//             -u     userid                     -> SYSAD
//             -p     password                   -> SYSAD
//             -d     delay between iterations   -> 0 seconds
//             -!     debug                      -> false
//             -?     Display settings only
//
//      --- synj0c3 started 11/06/98 12:34
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: CICSOS2
//
//      Iteration  Commarea Data (18)  Was/Is Length   CICS Rc   Abend
//      ---------  ------------------  -------------   -------   -----
//      1          11/06/98 13:34:59   32700   32714   0
//      1                              32700   32715   -1        AXTL
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- synj0c3 completed 11/06/98 12:35
//
//      --- Calls successful ... 15 out of 16
//      --- Iterations run ..... 1 out of 1
//
//      --- synj0c3 : largest commarea sent successfully: 32714
//p
//      The above example issues the first non-extended ECI synchronous call with
//      commarea length 32000, and then further ECI calls incrementing the
//      commarea length by one each time until an abend occurs (usually the
//      maximum commarea size, plus one.).
//      The commarea length is reset to the initial commarea length (-cmlen) upon
//      the start of each new iteration.
//p
//      The maximum commarea size varies according to the CICS server and
//      communication protocol being used (ie SNA, NetBIOS, TCP/IP or named pipes).
//p
//      Initial commarea size should be 18 or more characters in length
//      otherwise the commarea is not large enough to return a date and timestamp.
//p
//      History log:
//u
//l     Written : Jun 97 - Alan Hollingshead
//l     Updated : Apr 98 - new format using common classes
//
package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class synj0c3
{
   public static void main(String[] args)
   {
      int iBestCommLen    = 0;
      int iIt_Count       = 0;
      int iCallsIssued    = 0;
      int iReplyOK        = 0;
      int iCounter        = 0;
      String sInc_Comm    = " ";
      String sSpace       = "           ";

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nsynj0c3 - Detect Maximum Commarea Length") ;
      System.out.println("       -j     JGate                      -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port                 -> "+theseParms.sPort);
      System.out.println("       -list  list systems only          -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server                -> "+theseParms.sServer);
      System.out.println("       -cmlen initial commarea length    -> "+theseParms.iCommLen);
      System.out.println("       -+     increment commarea length  -> "+theseParms.bIncrementComm);
      System.out.println("       -i     # iterations               -> "+theseParms.sIterations);
      System.out.println("       -a     application                -> "+theseParms.sProgname);
      System.out.println("       -u     userid                     -> "+theseParms.sUserid);
      System.out.println("       -p     password                   -> "+theseParms.sPassword);
      System.out.println("       -d     delay between iterations   -> "+theseParms.sDelay+" seconds");
      System.out.println("       -!     debug                      -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java synj0c3 <parameters>");
         System.out.println(  "  e.g. java synj0c3 -s AlsKix -cmlen 32000 -+");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("synj0c3");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // CICS commarea - assume max value to be 65536 (64K)
      // most CICS servers can handle <=32K (minus fmh bits)
      byte[] Commarea = new byte[65536];

      // ------------------------------------------------------------------
      // Extended Contructor ECIRequest()
      // ------------------------------------------------------------------

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                               theseParms.sServer,
                                               theseParms.sUserid,
                                               theseParms.sPassword,
                                               theseParms.sProgname,
                                               theseParms.sTransid,
                                               Commarea,
                                               theseParms.iCommLen,
                                               0,
                                               0);

      String sGap1 = sSpace.substring(1,(12-(String.valueOf(theseParms.iIterations).length())));
      System.out.println("\nIteration  Commarea Data (18)  Was/Is Length   CICS Rc   Abend");
      System.out.println("---------  ------------------  -------------   -------   -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // set (or reset) initial commarea length
         eciRequest.Commarea_Length = theseParms.iCommLen;

         // start of do-while loop

         do
         {
            try
            {
               // set commarea to dashes
               for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
                  // reset commarea to dashes
                  eciRequest.Commarea[iCounter]=45;

               // flow ECI request
               iCallsIssued++;
               jgaConnection.flow(eciRequest);

               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
               {
                  iReplyOK++;
                  iBestCommLen = eciRequest.Commarea_Length;
               }
               else
               {
                  System.out.print("\n");
               }

               // for debug purposes
               T.ln(null,"Back again : AbendCode = " +
                                     eciRequest.Abend_Code);
               T.ln(null,"eci RC " + eciRequest.Cics_Rc);

               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea,0);

               String sGap2 = sSpace.substring(1,(12-(String.valueOf(iIt_Count).length())));
               String sGap3 = sSpace.substring(1,(9-(String.valueOf(theseParms.iCommLen).length())));
               String sGap4 = sSpace.substring(1,(9-(String.valueOf(eciRequest.Commarea_Length).length())));
               String sGap5 = sSpace.substring(1,(11-(String.valueOf(eciRequest.Cics_Rc).length())));
               // Display result
               System.out.print(iIt_Count + sGap2      +
                    ComData.substring(0,18) + "  "     +
                    theseParms.iCommLen + sGap3        +
                    eciRequest.Commarea_Length + sGap4 +
                    eciRequest.Cics_Rc + sGap5         +
                    eciRequest.Abend_Code + "    \r");

               }
               catch (Exception e)
               {
                  T.ln(null,"ERROR: Caught Exception: ");
                  T.ex(null,e);
                  System.out.println("\n--- Error returned from JGate flow ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
               eciRequest.Commarea_Length++;
            }

         while (theseParms.bIncrementComm && eciRequest.Cics_Rc
                  == ECIRequest.ECI_NO_ERROR);

         // end of do-while loop

         // start new line
         System.out.print("\n");

         // Delay next iteration if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            return;
         }
      // end iIt_Count for-loop
      }

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("synj0c3", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      String sMaxCommLen = (": largest commarea sent successfully: "
                            + (String.valueOf(iBestCommLen)));

      reportResults.reportComment("synj0c3", sMaxCommLen);

      System.exit(0);
   }
}
