//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//      synjms3 - Synchronous Calls to Multiple Servers
//p
//      This application issues non-extended ECI synchronous calls
//      to one, two, or three servers.
//p
//      The input parameters -s1, -s2, and -s3 enable up to three CICS servers
//      to be specified.
//x
//      Example: java jgtest.eci.synjms3 -s1 AlsKix -s2 IzKix -s3 ErKix
//p
//      This program contains no data conversion for the commarea.
//      Data conversion should be carried out on the server, if required.
//p
//      Parameters can be overridden via command line.
//p
//      Issue "synjms3 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.eci.synjms3 -s1 AlsKix -s2 cicsesa -s3 cics400 -i 3
//p
//      The above sample requests ECI synchronous calls to three different
//      CICS servers, iterating three times.
//p
//      Sample Output
//x
//      synjms3 - Synchronous Calls to Multiple Servers
//             -j      JGate              -> hollings
//             -r      JGate port         -> 2006
//             -list   list systems only  -> false
//             -s1     CICS server1       -> AlsKix
//             -s2     CICS server2       -> cicsesa
//             -s3     CICS server3       -> cics400
//             -a      application        -> SRVTIME
//             -tran   transid            -> CPMI
//             -i      # iterations       -> 3
//             -u      userid             -> SYSAD
//             -p      password           -> SYSAD
//             -d      delay              -> 0 seconds
//             -x      exit on ECI error  -> false
//             (fixed) eci_timeout value  -> 10 seconds
//             -!      debug              -> false
//             -?      Display settings only
//
//      --- synjms3 started 11/06/98 13:15
//
//      --- Successfully created JavaGateway connection ---
//
//      Iteration  Server    Commarea Data       CICS Rc   Abend
//      ---------  --------  -----------------   -------   -----
//      1          AlsKix    11/06/98 14:15:56   0
//      1          cicsesa   11/06/98 14:15:56   0
//      1          cics400                       -3        AZI4
//      2          AlsKix    11/06/98 14:15:57   0
//      2          cicsesa   11/06/98 14:15:57   0
//      2          cics400                       -3        AZI4
//      3          AlsKix    11/06/98 14:15:58   0
//      3          cicsesa   11/06/98 14:15:58   0
//      3          cics400                       -3        AZI4
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- synjms3 completed 11/06/98 13:15
//
//      --- Calls successful ... 6 out of 9
//      --- Iterations run ..... 3 out of 3
//p
//      History Log:
//u
//l     Written : July 97 - Alan Hollingshead
//l     Updated : May 98  - new package format
//

package jgtest.eci;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class synjms3
{
   static int iIt_Count             = 0;
   static int iCallsIssued          = 0;
   static int iReplyOK              = 0;
   static int iCounter              = 0;
   static JavaGateway jgaConnection = null;
   static boolean bExitNowYes       = false;
   static short  short10            = 10;
   static String sShort10           = String.valueOf(short10);

   public static void main(String[] args)
   {

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nsynjms3 - Synchronous Calls to Multiple Servers") ;
      System.out.println("       -j      JGate              -> "+theseParms.sJGate);
      System.out.println("       -r      JGate port         -> "+theseParms.sPort);
      System.out.println("       -list   list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -s1     CICS server1       -> "+theseParms.sServer1);
      System.out.println("       -s2     CICS server2       -> "+theseParms.sServer2);
      System.out.println("       -s3     CICS server3       -> "+theseParms.sServer3);
      System.out.println("       -a      application        -> "+theseParms.sProgname);
      System.out.println("       -tran   transid            -> "+theseParms.sTransid);
      System.out.println("       -i      # iterations       -> "+theseParms.sIterations);
      System.out.println("       -u      userid             -> "+theseParms.sUserid);
      System.out.println("       -p      password           -> "+theseParms.sPassword);
      System.out.println("       -d      delay              -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x      exit on ECI error  -> "+theseParms.bExitNow);
      System.out.println("       (fixed) eci_timeout value  -> "+sShort10+" seconds");
      System.out.println("       -!      debug              -> "+theseParms.bDebug);
      System.out.println("       -?      Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java synjms3 <parameters>");
         System.out.println(  "  e.g. java synjms3 -s1 AlsKix -s2 IzKix -s3 ErKix");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("synjms3");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer1.equals("(not specified)"))
      {
         theseParms.sServer1 = ECIlist.getServer1(jgaConnection);
      }

      String sSpace = "           ";
      String sGap1  = sSpace.substring(1,(12-(String.valueOf(theseParms.iIterations).length())));
      System.out.println("\nIteration  Server    Commarea Data       CICS Rc   Abend");
      System.out.println(  "---------  --------  -----------------   -------   -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // output banner every 15 iterations
         if (iIt_Count%15==0)
         {
            System.out.println("\nIteration  Server    Commarea Data       CICS Rc   Abend");
            System.out.println(  "---------  --------  -----------------   -------   -----");
         } // end-if

         try
         {
            // flow ECI request
            String sThisServer = theseParms.sServer1;
            eciSyncCall(sThisServer,theseParms);
            if (bExitNowYes) break iteration_loop;

            // flow ECI request to Server2 if specified
            if (theseParms.sServer2 != "(not specified)")
            {
               sThisServer = theseParms.sServer2;
               eciSyncCall(sThisServer,theseParms);
               if (bExitNowYes) break iteration_loop;
            }

            // flow ECI request to Server3 if specified
            if (theseParms.sServer3 != "(not specified)")
            {
               sThisServer = theseParms.sServer3;
               eciSyncCall(sThisServer,theseParms);
               if (bExitNowYes) break iteration_loop;
            }

         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error returned from JGate flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Delay next iteration if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            return;
         }
         // end iIt_Count for-loop
      }

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("synjms3", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }

   //----------------------------------------------------------------------
   // Method : eciSyncCall
   // Issue an ECI_SYNC call for this server
   //----------------------------------------------------------------------

   protected static boolean eciSyncCall(String sThisServer,
                                     inputParms theseParms ) throws Exception
   {
      // CICS Commarea
      byte[] Commarea = new byte[18];

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                               sThisServer,
                                               theseParms.sUserid,
                                               theseParms.sPassword,
                                               theseParms.sProgname,
                                               theseParms.sTransid,
                                               Commarea,
                                               theseParms.iCommLen,
                                               ECIRequest.ECI_NO_EXTEND,
                                               ECIRequest.ECI_LUW_NEW );

      // set ECI timeout to prevent a hang from a non-responding server
      eciRequest.setECITimeout(short10);

      // set commarea to dashes
      for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
         // reset commarea to dashes
         eciRequest.Commarea[iCounter]=45;

      iCallsIssued++;
      jgaConnection.flow(eciRequest);

      // for debug purposes
      T.ln(null,"Back again : AbendCode = " +
                            eciRequest.Abend_Code);
      T.ln(null,"eci RC " + eciRequest.Cics_Rc);

      // Convert ASCII bytes to ASCII characters
      String ComData  = new String(eciRequest.Commarea,0);

      String sSpace10 = "          ";
      String sSpace13 = "             ";
      String sSpace12 = "            ";
      String sGap2 = sSpace13.substring(1,(12-(String.valueOf(iIt_Count).length())));
      String sGap3 = sSpace12.substring(1,(11-(sThisServer.length())));
      String sGap4 = sSpace12.substring(1,(11-(String.valueOf(eciRequest.Cics_Rc).length())));
      // Display result
      System.out.println(iIt_Count + sGap2 +
           sThisServer             + sGap3 +
           ComData.substring(0,18) + "  "  +
           eciRequest.Cics_Rc      + sGap4 +
           eciRequest.Abend_Code +"    ");

      if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
      {
         iReplyOK++;
      }
      else
      {
         if (theseParms.bExitNow)
         {
            System.out.println("\n--- ECI CICS_RC error - aborting program");
            bExitNowYes = true;
         }
      }
      return bExitNowYes;
   }
}
