//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//3     epihceci - EPI (HLI) Issuing CECI Commands
//
//p     This application uses the Higher Level Interface (HLI) to the EPI
//      to issue a CECI LINK PROGRAM command.
//
//p     The order of commands is:
//o
//l     Connect a terminal to CICS
//l     Enter a CECI LINK PROGRAM command
//l     Execute the CECI command
//l     Exit CECI command
//l     Disconnect terminal from CICS
//
//p     The default program is SRVTIME which returns the time and datestamp
//      in the commarea to the client application.
//
//p     Parameters can be overridden via command line.
//
//p     It is important that a model terminal definition suitable for
//      a CICS Client is available on the CICS server
//
//p     For example, use:
//u
//l     DFLT on CICS for OS/2
//l     ibm-cics-client on CICS for AIX
//l     MT1A on CICS for OS/390 (Hursley Test Systems)
//
//p     Issue "ecihceci -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.epi.epihceci -j local: -m ibm-cics-client
//p
//      Sample Output
//x
//      epihceci - EPI (HLI) Issuing CECI Commands
//             -j     JGate              -> local:
//             -r     JGate port         -> 2006
//             -list  list systems only  -> false
//             -model model/device type  -> ibm-cics-client
//             -s     CICS server        -> (not specified)
//             -a     application        -> SRVTIME
//             -i     # iterations       -> 1
//             -u     userid             -> SYSAD
//             -p     password           -> SYSAD
//             -d     delay              -> 0 seconds
//             -x     exit on EPI error  -> false
//             -!     debug              -> false
//             -?     Display settings only
//
//      --- epihceci started 18/06/98 17:38
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      ---------------------------------------------------
//      epihceci --- Iteration: 1 of 1
//      ---------------------------------------------------
//
//      --> Connect terminal to CICS
//          NetName .................. CL01C000
//          DeviceType ............... null
//          Connected to server ...... NETBTCLR
//          Terminal state ........... idle
//          Associated screen ........ ibm.cics.jgate.epi.Screen@17f4
//          Current transaction ...... null
//          ATI state ................ disabled
//      --> Enter CECI
//          Current transaction ...... null
//          CECI command ............. CECI LINK PROGRAM(SRVTIME) COMM(&DFHW) LEN(18)
//          CECI Status .............. About to execute command
//          Old commarea (18 chars) .. WRITEQ TS QUEUE(CE
//      --> Hit <ENTER>
//          Current transaction ...... null
//          CECI Status .............. Command execution complete
//          New commarea (18 chars) .. 18/06/98 17:38:20.
//          CICS Response field ...... Normal
//      --> Exit via PF3
//      --> Disconnect EPI session from CICS
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- epihceci completed 18/06/98 17:38
//
//      --- Calls successful ... 1 out of 1
//      --- Iterations run ..... 1 out of 1
//
//p
//      History log:
//u
//l     Written : Dec 97 - Alan Hollingshead
//l     Updated : Apr 98 - uses common classes
//

package jgtest.epi;

import ibm.cics.jgate.epi.*;
import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class epihceci
{
   public static void main(String[] args)
   {
      int iIt_Count       = 0;
      int iCallsIssued    = 0;
      int iReplyOK        = 0;
      String sField       = null;
      String sOldCommarea = null;
      String sNewCommarea = null;

      //  CICS commarea
      byte[] Commarea = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nepihceci - EPI (HLI) Issuing CECI Commands") ;
      System.out.println("       -j     JGate              -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port         -> "+theseParms.sPort);
      System.out.println("       -list  list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -model model/device type  -> "+theseParms.sModel);
      System.out.println("       -s     CICS server        -> "+theseParms.sServer);
      System.out.println("       -a     application        -> "+theseParms.sProgname);
      System.out.println("       -i     # iterations       -> "+theseParms.sIterations);
      System.out.println("       -u     userid             -> "+theseParms.sUserid);
      System.out.println("       -p     password           -> "+theseParms.sPassword);
      System.out.println("       -d     delay              -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x     exit on EPI error  -> "+theseParms.bExitNow);
      System.out.println("       -!     debug              -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java epihceci <parameters>");
         System.out.println(  "  e.g. java epihceci -s AlsKix -model ibm-cics-client");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("epihceci");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class EPIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class EPIlist to list systems available
         EPIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = EPIlist.getServer1(jgaConnection);
      }

      // ----------------------------------------------------------------------
      // Main EPI Code
      // ----------------------------------------------------------------------

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // display iteration header
         System.out.println("\n---------------------------------------------------");
         System.out.println("epihceci --- Iteration: "+iIt_Count+" of "
                            +theseParms.iIterations);
         System.out.println("---------------------------------------------------");
         try {
             System.out.println( "\n--> Connect terminal to CICS" );

             Terminal terminal = new Terminal( jgaConnection, theseParms.sServer, null, null );

             // call class to display terminal details
             HEPIUtil.showTerminalStatus(terminal);

             System.out.println( "--> Enter CECI ");

             String sCECIcmd = "LINK PROGRAM(" + theseParms.sProgname + ") COMM(&DFHW) LEN(18)";
             terminal.send( null, "CECI", sCECIcmd );

             Screen screen = terminal.getScreen();

             System.out.println( "    Current transaction ...... " + terminal.getTransID() );
             System.out.println( "    CECI command ............. CECI " + sCECIcmd );

             // Output STATUS: and Commarea fields
             for ( int i=1; i <= screen.fieldCount(); i++ ) {
                 if ( screen.field(i).textLength() > 0 ) {
                     sField = screen.field(i).getText().toUpperCase();

                     // check for STATUS
                     if ( sField.indexOf("STATUS") > -1 ) {
                        i++;
                        System.out.println( "    CECI Status .............. " + screen.field(i).getText() );
                     }

                     // check for Commarea
                     if ( sField.indexOf("COMMAREA") > -1 ) {
                        i++;
                        System.out.println( "    Old commarea (18 chars) .. " + screen.field(i).getText().substring(0,18) );
                        sOldCommarea = screen.field(i).getText().substring(0,18);
                     }

                 }
             }

             screen.setAID( AID.enter );

             System.out.println( "--> Hit <ENTER>");
             terminal.send();

             // System.out.println( "Processing CICS reply" );

             screen = terminal.getScreen();

             System.out.println( "    Current transaction ...... " + terminal.getTransID() );

             // Output STATUS: Commarea and RESPONSE fields
             for ( int i=1; i <= screen.fieldCount(); i++ ) {
                 if ( screen.field(i).textLength() > 0 ) {
                     sField = screen.field(i).getText().toUpperCase();

                     // check for STATUS
                     if ( sField.indexOf("STATUS") > -1 ) {
                        i++;
                        System.out.println( "    CECI Status .............. " + screen.field(i).getText() );
                     }

                     // check for Commarea
                     if ( sField.indexOf("COMMAREA") > -1 ) {
                        i++;
                        System.out.println( "    New commarea (18 chars) .. " + screen.field(i).getText().substring(0,18) );
                        sNewCommarea = screen.field(i).getText().substring(0,18);
                     }

                     // check for RESPONSE
                     if ( sField.indexOf("RESPONSE") > -1 ) {
                        i++;
                        // display string response only, not eibresp
                        int iRespLen = screen.field(i).getText().length();
                        if (iRespLen > 12) iRespLen=12;
                        System.out.println( "    CICS Response field ...... " + screen.field(i).getText().substring(0,iRespLen) );
                     }
                 }
             }

             // check for timestamped response
             if (!sOldCommarea.equals(sNewCommarea))
                iReplyOK++;

             screen.setAID( AID.PF3 );

             System.out.println( "--> Exit via PF3");
             terminal.send();

             System.out.println( "--> Disconnect EPI session from CICS");
             terminal.disconnect();

         } /* endtry */
         catch ( Exception e  )
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error returned from JGate flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }
      }  // end iIt_Count for-loop

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      // Calls_Issued replaced by Iteration_Count for this program
      --iIt_Count;
      reportResults.reportECIresults("epihceci", iReplyOK, iIt_Count,
                                     iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
