//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//3     epij123 - Traditional Java EPI Issuing CECI Commands
//
//p     This application uses traditional Java EPI calls (rather than
//      the Higher Level Interface (HLI) to the EPI as used in epihceci)
//      to issue a CECI LINK PROGRAM command.
//
//p     The program runs as follows:
//o
//l     addTerminal
//l     startTran CECI with data "link program(srvtime) comm(&dfhw) len(18)"
//l     getEvent
//l     reply <enter>
//l     getEvent
//l     extract from returned data the date and timestamp.
//l     reply <F3>
//l     getEvent
//l     delTerminal
//l     getEvent
//
//p     The default program is SRVTIME which returns the time and datestamp
//      in the commarea to the client application.
//
//p     Parameters can be overridden via command line.
//
//p     It is important that a model terminal definition suitable for
//      a CICS Client is available on the CICS server. For example:
//p
//D     CICS for OS/2 server
//d        DFLT (a supplied model terminal definition)
//
//D     CICS for AIX server
//d        ibm-cics-client (a pre-defined model terminal definition)
//
//D     CICS for OS/390 (Hursley Test Systems)
//d        MT1A (a model terminal definition that requires defining)
//
//p     Issue "epij123 -?" for help and default settings.
//p
//      Sample Input
//x
//      java jgtest.epi.epij123 -j local: -s AlsKix -model ibm-cics-client
//p
//      Sample Output
//x
//      epij123 - Traditional Java EPI Issuing CECI Commands
//             -j        JGate              -> local:
//             -r        JGate port         -> 2006
//             -list     list systems only  -> false
//             -model    model/device type  -> ibm-cics-client
//             -s        CICS server        -> AlsKix
//             -a        application        -> SRVTIME
//             -i        # iterations       -> 1
//             -u        userid             -> SYSAD
//             -p        password           -> SYSAD
//             -d        delay              -> 0 seconds
//             -verbose  verbose            -> false
//             -!        debug              -> false
//             -?        Display settings only
//
//      --- epij123 started 23/07/98 12:05
//
//      --- Successfully created JavaGateway connection ---
//
//      ---------------------------------------------------
//          epij123 --- Iteration: 1 of 1
//      ---------------------------------------------------
//      -->  epi_add_terminal     0   Netname: SQAAAAHH
//      -->  epi_start_tran       0   CECI LINK PROG(SRVTIME) ...
//      -->  epi_get_event       17   epi_event_send
//      -->  epi_get_event        0   epi_event_send
//      -->  epi_get_event        0   epi_event_converse
//      -->  epi_reply            0   <enter>
//      -->  epi_get_event       17   epi_event_send
//           Data extracted -->       Commarea: 23/07/98 13:05:17
//      -->  epi_get_event        0   epi_event_send
//      -->  epi_get_event        0   epi_event_converse
//      -->  epi_reply            0   <F3> to exit CECI
//      -->  epi_get_event       17   epi_event_send
//      -->  epi_get_event        0   epi_event_send
//      -->  epi_get_event        0   epi_event_end_tran
//      -->  epi_del_terminal     0   Netname: SQAAAAHH
//      -->  epi_get_event        0   epi_event_end_term
//      -->  endReason            0   epi_end_signoff
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- epij123 completed 23/07/98 12:05
//
//      --- Calls successful ... 1 out of 1
//      --- Iterations run ..... 1 out of 1
//
//
//p
//      History log:
//u
//l     Written : Apr 97 - Alan Hollingshead
//l     Updated : Jul 98 - uses common classes
//
//

package jgtest.epi;

import ibm.cics.jgate.epi.*;
import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class epij123
{
   static boolean bCommFnd    = false;
   static boolean bExit       = false;
   static boolean bLoop       = true;

   public static void main(String[] args)
   {
      int iIt_Count           = 0;
      int iCallsIssued        = 0;
      int iReplyOK            = 0;
      String sField           = null;
      String sOldCommarea     = null;
      String sNewCommarea     = null;
      String sNetName         = "";

      //  CICS commarea
      byte[] Commarea         = new byte[18];

      boolean bDelTerm        = true;
      int     iIterationDone  = 0;
      int     iATIState       = 0;
      int     iSize           = 3;
      int     iWaitState      = EPIRequest.EPI_WAIT;
      boolean bDataOK         = false ;
      byte[]  abytData        = new byte[20];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nepij123 - Traditional Java EPI Issuing CECI Commands") ;
      System.out.println("       -j        JGate              -> "+theseParms.sJGate);
      System.out.println("       -r        JGate port         -> "+theseParms.sPort);
      System.out.println("       -list     list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -model    model/device type  -> "+theseParms.sModel);
      System.out.println("       -s        CICS server        -> "+theseParms.sServer);
      System.out.println("       -a        application        -> "+theseParms.sProgname);
      System.out.println("       -i        # iterations       -> "+theseParms.sIterations);
      System.out.println("       -u        userid             -> "+theseParms.sUserid);
      System.out.println("       -p        password           -> "+theseParms.sPassword);
      System.out.println("       -d        delay              -> "+theseParms.sDelay+" seconds");
      System.out.println("       -verbose  verbose            -> "+theseParms.bVerbose);
//    System.out.println("       -x        exit on EPI error  -> "+theseParms.bExitNow);
      System.out.println("       -!        debug              -> "+theseParms.bDebug);
      System.out.println("       -?        Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java epij123 <parameters>");
         System.out.println(  "  e.g. java epij123 -s AlsKix -model ibm-cics-client -verbose");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("epij123");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class EPIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class EPIlist to list systems available
         EPIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = EPIlist.getServer1(jgaConnection);
      }

      // ----------------------------------------------------------------------
      // Main EPI Code
      // ----------------------------------------------------------------------

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // display iteration header
         System.out.println("\n---------------------------------------------------");
         System.out.println("    epij123 --- Iteration: "+iIt_Count+" of "
                            +theseParms.iIterations);
         System.out.println("---------------------------------------------------");
         try
         {
            // ----------------------------------------------------------------
            // add Terminal
            // ----------------------------------------------------------------
            EPIRequest epiRequest = EPIRequest.addTerminal(theseParms.sServer,
                                    sNetName, theseParms.sModel);
            TermAdd(jgaConnection,epiRequest,theseParms);
            System.out.println("-->  epi_add_terminal     "+
                               epiRequest.Cics_Rc + "   " + "Netname: "+
                               epiRequest.netName );
            if (bExit) break iteration_loop;

            if (epiRequest.Cics_Rc != EPIRequest.EPI_NORMAL)
            {
               System.out.println("ERROR: check server log and model/netname definitions");
               break iteration_loop;
            }

            // ----------------------------------------------------------------
            // start CECI
            // ----------------------------------------------------------------
            epiRequest.startTran(theseParms.sTransid,abytData,iSize);
            startCECI(jgaConnection,epiRequest,theseParms.sProgname);
            System.out.println("-->  epi_start_tran       "+
                           epiRequest.Cics_Rc + "   CECI LINK PROG(" +
                           theseParms.sProgname + ") ...");
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // getEventLoop - 1
            // ----------------------------------------------------------------
            getEventLoop(jgaConnection,epiRequest,iWaitState,theseParms);
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // epi_sendReply to execute CECI command via <enter>
            // ----------------------------------------------------------------
            replyEnter(jgaConnection,epiRequest);
            System.out.println("-->  epi_reply            "+
                               epiRequest.Cics_Rc + "   <enter> " );
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // getEventLoop - 2
            // ----------------------------------------------------------------
            getEventLoop(jgaConnection,epiRequest,iWaitState,theseParms);
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // check commarea has been found else put out warning
            // ----------------------------------------------------------------
            if (bCommFnd == false)
            {
               System.out.println("     Commarea NOT found! Check data flows with -verbose option");
            }
            else
            {
               iReplyOK++;
            }
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // epi_sendReply to exit CECI via <F3>
            // ----------------------------------------------------------------
            replyPF3(jgaConnection,epiRequest);
            System.out.println("-->  epi_reply            "+
                               epiRequest.Cics_Rc + "   <F3> to exit CECI" );
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // getEventLoop
            // ----------------------------------------------------------------
            getEventLoop(jgaConnection,epiRequest,iWaitState,theseParms);
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // delete Terminal
            // ----------------------------------------------------------------
            epiRequest.delTerminal();
            TermDel(jgaConnection,epiRequest);
            System.out.println("-->  epi_del_terminal     "+
                               epiRequest.Cics_Rc + "   " + "Netname: "+
                               epiRequest.netName );
            if (bExit) break iteration_loop;

            // ----------------------------------------------------------------
            // getEventLoop
            // ----------------------------------------------------------------
            getEventLoop(jgaConnection,epiRequest,iWaitState,theseParms);
            if (bExit) break iteration_loop;

            String sEndReason;
            if (epiRequest.endReason == 1)
            {
               sEndReason = "epi_end_signoff";
            }
            else
            {
               sEndReason = "Unexpected reason "+epiRequest.endReason;
            }
            System.out.println("-->  endReason            "+
                               epiRequest.Cics_Rc + "   " + sEndReason );

            iIterationDone++;
         // ------------------------------------------------------------------------
         // Iteration loop ends here ...
         // ------------------------------------------------------------------------
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during iteration "+ iIt_Count +" ---\n");
            System.out.println("--- Caught Exception: " + e);
         }
      } // end of iteration-loop
   // ----------------------------------------------------------------------
   // Call class closeJGate to close Java Gateway connection and exit
   // ----------------------------------------------------------------------

   setJGate.closeJGate(jgaConnection);

   // ----------------------------------------------------------------------
   // Call class reportResults to display results
   // ----------------------------------------------------------------------

   // Calls_Issued replaced by Iteration_Count for this program
   iIt_Count--;
   reportResults.reportECIresults("epij123", iReplyOK, iIt_Count,
                                  iIt_Count, theseParms.iIterations);

   System.exit(0);

   } // end-run() method

   //--------------------------------------------------------------------------
   //-------- Now for the individual EPI methods ------------------------------
   //--------------------------------------------------------------------------
   //
   //--------------------------------------------------------------------------
   // addTerminal method
   //--------------------------------------------------------------------------
   public static void TermAdd(JavaGateway jgaConnection,
                              EPIRequest epiRequest, inputParms theseParms)
   {
      try {
         jgaConnection.flow(epiRequest);
         T.ln(null,"******addTerminal RC = " + epiRequest.Cics_Rc);
         T.ln(null,"******addTerminal termIndex = " + epiRequest.termIndex);

         if (theseParms.bVerbose == true)
         {
             System.out.println("--- addTerminal results ---");
             System.out.println("    Cics_Rc ........ "+epiRequest.Cics_Rc);
             System.out.println("    termIndex ...... "+epiRequest.termIndex);
             System.out.println("    netName ........ "+epiRequest.netName);
             System.out.println("    numLines ....... "+epiRequest.numLines);
             System.out.println("    numColumns ..... "+epiRequest.numColumns);
             System.out.println("    maxData ........ "+epiRequest.maxData);
             System.out.println("    errLastLine .... "+epiRequest.errLastLine);
             System.out.println("    errIntensify ... "+epiRequest.errIntensify);
             System.out.println("    errColor ....... "+epiRequest.errColor);
             System.out.println("    errHilight ..... "+epiRequest.errHilight);
             System.out.println("    hilight ........ "+epiRequest.hilight);
             System.out.println("    color .......... "+epiRequest.color);
             System.out.println("    printer ........ "+epiRequest.printer);
             System.out.println("--- --- --- --- --- --- ---");
         }
      }
      catch (Exception e)
      {
         T.ln(null,"ERROR: Caught Exception: ");
         T.ex(null,e);
         System.out.println("\n--- Error during addTerminal() ---\n");
         System.out.println("--- Caught Exception: " + e);
         bExit = true;
         return;
      }
   }

   //--------------------------------------------------------------------------
   // NumSystems method
   //--------------------------------------------------------------------------
   public static void NumSystems(JavaGateway jgaConnection,
                      EPIRequest epiRequest, inputParms theseParms)
   {
      try
      {
         jgaConnection.flow(epiRequest);
         T.ln(null,"********listSystems RC " + epiRequest.Cics_Rc);
         if (theseParms.bVerbose == true)
         {
            System.out.println("---  Available Servers  ---");
            if (epiRequest.getRc() == 0)
            {
               for (int iSysLoop = 0;
                  iSysLoop < epiRequest.SystemList.size();
                  iSysLoop += 2)
                  {
                     System.out.println("    System : " +
                             epiRequest.SystemList.elementAt(iSysLoop) +
                             ", Description : " +
                             epiRequest.SystemList.elementAt(iSysLoop + 1));
               } // end-for
            } // end-if
            else
            {
            System.out.println("EPIRequest.listSystems failed with Rc : " +
                                epiRequest.getRc());
            } // end-else
         System.out.println("--- --- --- --- --- --- ---");
         } // end-if
      } // end-try
      catch (Exception e)
      {
         T.ln(null,"ERROR: Caught Exception: ");
         T.ex(null,e);
         System.out.println("\n--- Error during NumSystems() ---\n");
         System.out.println("--- Caught Exception: " + e);
         bExit = true;
         return;
      }
   }

   //--------------------------------------------------------------------------
   // TermDel method
   //--------------------------------------------------------------------------
   public static void TermDel(JavaGateway jgaConnection, EPIRequest epiRequest)
   {
      try
      {
         if (jgaConnection != null)
         {
            jgaConnection.flow(epiRequest);
            T.ln(null,"********delTerminal RC " + epiRequest.Cics_Rc);
         }
      }
      catch (Exception e)
      {
         T.ln(null,"ERROR: Caught Exception: ");
         T.ex(null,e);
         System.out.println("\n--- Error during TermDel() ---\n");
         System.out.println("--- Caught Exception: " + e);
         bExit = true;
         return;
      }
   }

   //--------------------------------------------------------------------------
   // getEventLoop method
   //--------------------------------------------------------------------------
   public static void getEventLoop(JavaGateway jgaConnection,
                EPIRequest epiRequest, int iWaitState, inputParms theseParms)
  {
      boolean bLoop = true;
      int iSize     = 1900;
      String sEvent = "";
      // flow getEvent
      epiRequest.getEvent(iWaitState,iSize);
      try
      {
         jgaConnection.flow(epiRequest);
      }
      catch (Exception e)
      {
         T.ln(null,"ERROR: Caught Exception: ");
         T.ex(null,e);
         System.out.println("\n--- Error during getEventLoop() ---\n");
         System.out.println("--- Caught Exception: " + e);
         bExit = true;
         return;
      }
      // need to recode this loop if epi_nowait supported as an event
      // may not be returned immediately (ie Cics_Rc = 16)
      while (bLoop)
      {
         switch(epiRequest.event)
         {
            case EPIRequest.EPI_EVENT_SEND :
                           sEvent = "epi_event_send";
                           // check cics_rc to line up output
                           if (epiRequest.Cics_Rc > 9)
                           {
                              System.out.println("-->  epi_get_event       "+
                                              epiRequest.Cics_Rc + "   " + sEvent );
                           }
                           else
                           {
                              System.out.println("-->  epi_get_event        "+
                                              epiRequest.Cics_Rc + "   " + sEvent );
                           }
                           if (theseParms.bVerbose == true)
                           {
                              if (epiRequest.size != 0)
                              {
                                 System.out.println("--- --- --- --- --- --- ---");
                                 System.out.println("--- 3270 datastream (length: "+ epiRequest.size + ") --- --- --- ---");
                                 String ComData = new String(epiRequest.data,0);
                                 ComData = ComData.trim();
                                 System.out.println(ComData);
                                 System.out.println("--- end of 3270 datastream  --- --- --- --- --- ---");
                              }
                           }
                           // check data if some returned
                           // set Commarea Data Found flag (bCommFnd) false
                           bCommFnd = false;
                           if (epiRequest.size != 0)
                              checkData(epiRequest);

                           // -------------------------------------------------
                           // flow another getEvent
                           // -------------------------------------------------
                           epiRequest.getEvent(iWaitState,iSize);
                           try
                           {
                              jgaConnection.flow(epiRequest);
                           }
                           catch (Exception e)
                           {
                              T.ln(null,"ERROR: Caught Exception: ");
                              T.ex(null,e);
                              System.out.println("\n--- Error during getEventLoop() ---\n");
                              System.out.println("--- Caught Exception: " + e);
                              bExit = true;
                              return;
                           }
                           break;
            case EPIRequest.EPI_EVENT_CONVERSE :
                           if ((epiRequest.size > 0) && (epiRequest.data[0] == 0x32))
                           {
                              // -------------------------------------------------
                              // reply with Enter
                              // -------------------------------------------------
                              replyEnter(jgaConnection,epiRequest);
                              System.out.println("-->  epi_reply            "+
                                                 epiRequest.Cics_Rc + "   <enter> " );
                              // -------------------------------------------------
                              // flow another getEvent
                              // -------------------------------------------------
                              epiRequest.getEvent(iWaitState,iSize);
                              try
                              {
                                 jgaConnection.flow(epiRequest);
                              }
                              catch (Exception e)
                              {
                                 T.ln(null,"ERROR: Caught Exception: ");
                                 T.ex(null,e);
                                 System.out.println("\n--- Error during getEventLoop() ---\n");
                                 System.out.println("--- Caught Exception: " + e);
                                 bExit = true;
                                 return;
                              }
                              bLoop = true;
                           }
                           else
                           {
                              sEvent = "epi_event_converse";
                              bLoop = false;
                           }
                           break;
            case EPIRequest.EPI_EVENT_END_TRAN :
                           sEvent = "epi_event_end_tran";
                           bLoop = false;
                           break;
            case EPIRequest.EPI_EVENT_END_TERM :
                           sEvent = "epi_event_end_term";
                           bLoop = false;
                           break;
            default :
                           System.out.println("ERROR: unexpected event");
                           System.out.println("Cics_Rc : " + epiRequest.Cics_Rc);
                           System.out.println("Event returned: "+epiRequest.event);
                           return;
         }
         // check cics_rc to line up output
         if (epiRequest.Cics_Rc > 9)
         {
            System.out.println("-->  epi_get_event       "+
                            epiRequest.Cics_Rc + "   " + sEvent );
         }
         else
         {
            System.out.println("-->  epi_get_event        "+
                            epiRequest.Cics_Rc + "   " + sEvent );
         }
      } // end-while
   }

   //--------------------------------------------------------------------------
   // startCECI method
   //--------------------------------------------------------------------------
   public static void startCECI(JavaGateway jgaConnection, EPIRequest epiRequest, String sApp)
   {
      //
      // Issue: ceci link prog(<APPLICATION>) comm(&dfhw) len(18)
      //
      // set up local variables
      String sTransid     = null;
      int iSize       = 46;

      // 3270 datastream in ASCII
      String epiString = "CECI LINK PROG(" + sApp + ") COMM(&DFHW) LEN(18)";

      // byteArray contains the 'epiString' converted to byte values
      byte[] byteArray = new byte[(epiString.length()+3)];

      // insert 3 byte header
      byteArray[0] = 0x27;
      // buffer address - row 1 col 1
      byteArray[1] = 0x20;
      byteArray[2] = 0x20;

      // convert epiString to byte values in 'byteArray'
      epiString.getBytes(0,epiString.length(),byteArray,3);

      // flow CECI command
      try
      {
         epiRequest.startTran(sTransid,byteArray,iSize);
         jgaConnection.flow(epiRequest);
         T.ln(null,"******startTran (CECI cmd) RC " + epiRequest.Cics_Rc +
                                     " ,termIndex = " + epiRequest.termIndex);
      }
      catch (Exception e)
      {
                T.ln(null,"ERROR: Caught Exception: ");
                T.ex(null,e);
                System.out.println("\n--- Error during startCESN() ---\n");
                System.out.println("--- Caught Exception: " + e);
                bExit = true;
                return;
      }
   }

   //--------------------------------------------------------------------------
   // replyPF3 method
   //--------------------------------------------------------------------------
   public static void replyPF3(JavaGateway jgaConnection, EPIRequest epiRequest)
   {
      byte[] abytData = new byte[3];
      int iSize       = 3;
      // Create datastream for CICS server to read - ASCII platform - OS/2
      //
      // To calculate screen address:
      //
      // AID    = 0x33 (PF3)
      //          Find this value in DFHAID.H  where DFHPF3= '3'
      //          where '_' equates to 0x33 in CICS3270.C
      // SBA    = not required
      // Cursor = not required
      //
      abytData[0]  = 0x33;
      abytData[1]  = 0x20;
      abytData[2]  = 0x20;
      try
      {
         epiRequest.sendReply(abytData,iSize);
         jgaConnection.flow(epiRequest);
         T.ln(null,"**sendReply of PF3 to exit txn) RC " + epiRequest.Cics_Rc +
                                " ,termIndex = " + epiRequest.termIndex);
      }
      catch (Exception e)
      {
         T.ln(null,"ERROR: Caught Exception: ");
         T.ex(null,e);
         System.out.println("\n--- Error during sendReply() ---\n");
         System.out.println("--- Caught Exception: " + e);
         bExit = true;
         return;
      }
   }

   //--------------------------------------------------------------------------
   // replyEnter method
   //--------------------------------------------------------------------------
   public static void replyEnter(JavaGateway jgaConnection, EPIRequest epiRequest)
   {
      byte[] abytData = new byte[3];
      int iSize       = 3;
      //
      // AID    = 0x27 <Enter>
      // SBA    = not required
      // Cursor = not required
      //
      abytData[0]  = 0x27;
      abytData[1]  = 0x20;
      abytData[2]  = 0x20;
      try
      {
         epiRequest.sendReply(abytData,iSize);
         jgaConnection.flow(epiRequest);
         T.ln(null,"**sendReply of Enter to execute command) RC " + epiRequest.Cics_Rc +
                                " ,termIndex = " + epiRequest.termIndex);
      }
      catch (Exception e)
      {
         T.ln(null,"ERROR: Caught Exception: ");
         T.ex(null,e);
         System.out.println("\n--- Error during replyEnter() ---\n");
         System.out.println("--- Caught Exception: " + e);
         bExit = true;
         return;
      }
   }

   //--------------------------------------------------------------------------
   // checkData method
   //--------------------------------------------------------------------------
   public static void checkData(EPIRequest epiRequest)
   {
      if (epiRequest.size != 0)
      {
         String ComData = new String(epiRequest.data,0);
         // uppercase the data
         ComData = ComData.toUpperCase();
         if (ComData.indexOf("ABOUT") > -1)
         {
            // found 'ABOUT TO EXECUTE COMMAND so do nothing
         }
         else if (ComData.indexOf("COMPLETE") > -1)
         {
            // found word 'COMPLETE' so extract commarea data
            // ie date and timestamp from SRVTIME
            // Now find "COMMAREA" as slash appears before this!
            int iCommarea = (ComData.indexOf("COMMAREA"));
            // Now find '/' slash in datastamp, then back two chars
            // starting from "COMMAREA" position.
            int iComDataStart = (ComData.indexOf("/",iCommarea) -2);
            // Commarea data is 17 chars long.
            ComData = ComData.substring(iComDataStart,iComDataStart+17);
            System.out.println("     Data extracted -->       "+
                               "Commarea: " + ComData );
            bCommFnd = true;
         }
      }
   }
   //--------------------------------------------------------------------------
}  // end-class
