//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//3     cpgj001 - CICS Client Codepage Details
//
//p     This application displays the following information:
//u
//l     CICS Client Codepage Details
//l     Java Gateway Information
//l     Operating System Properties
//l     Java Properties
//
//p     The program does not require an active connection to a server.
//
//p     Parameters can be overridden via command line.
//
//p     Issue "cpgj001 -?" for help and default settings.
//
//p     Sample Input
//
//x     java jgtest.gateway.cpgj001 -j local:
//
//p     Sample Output
//
//x
//      cpgj001 - CICS Client Codepage Details
//             -j     JGate              -> local:
//             -r     JGate port         -> 2006
//             -list  list systems only  -> false
//             -!     debug              -> false
//             -?     Display settings only
//
//      --- cpgj001 started 18/06/98 14:43
//
//      --- Successfully created JavaGateway connection ---
//
//      CICS Client Codepage Details:
//      CICS Client Codepage .............. unknown
//      CICS Client Return Code ........... 0
//      CICS Client Return Code (string) .. CODEPAGE_NORMAL
//
//      Java Gateway:
//      Java Gateway Return Code .......... 0
//
//      System Properties:
//      Operating System .................. OS/2
//      Operating System Version .......... 20.40
//      Language Code ..................... en
//      Country Code ...................... GB
//      Time Zone ......................... gmt
//
//      Java Properties:
//      JVM locale ........................ IBM-850
//      Local encodings to Unicode package. sun.io
//      Java Interpreter .................. JDK 1.1.1 IBM build o111-19970726
//      Java Installation Directory ....... D:\JAVA11\BIN\..
//      JIT Compiler ...................... javax
//      Java Classpath .................... e:\j_reuse;c:\www\cgi-bin\icsclass.zip;d:\javaos2\lib\classes.zip;d:\netscape\njclass.zip;d:\jgate\classes;.\.;D:\JAVA11\DLL\..\classes;D:\JAVA11\DLL\..\lib\classes.zip;D:\JAVA11\DLL\..\classes;
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- cpgj001 completed 18/06/98 14:43
//
//      --- Iterations run ..... 1 out of 1
//
//
//p     History log:
//u
//l     Written : Nov 97 - Alan Hollingshead (based on v1.1.3 version by Chris Goodall)
//l     Updated : Apr 98 - uses common classes
//
//
package jgtest.gateway;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class cpgj001
{
   public static void main(String[] args)
   {

      // CICS request object
      CicsCpRequest cpReq = new CicsCpRequest();

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\ncpgj001 - CICS Client Codepage Details");
      System.out.println("       -j     JGate              -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port         -> "+theseParms.sPort);
      System.out.println("       -list  list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -!     debug              -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java cpgj001 <parameters>");
         System.out.println(  "  e.g. java cpgj001 -j AlsJGate -!");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("cpgj001");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Codepage details
      // ----------------------------------------------------------------------

      System.out.println("\nCICS Client Codepage Details:");
      System.out.println("CICS Client Codepage .............. " + cpReq.getClientCp());
      System.out.println("CICS Client Return Code ........... " + cpReq.getClientRc());
      System.out.println("CICS Client Return Code (string) .. " + cpReq.getClientRcString());

      // not supported in CG4J v2.0+
      // System.out.println("Server Cp = " + cpReq.getServerCp());
      // System.out.println("Server Rc = " + cpReq.getServerRcString());

      System.out.println("\nJava Gateway:");
      System.out.println("Java Gateway Return Code .......... " + cpReq.getRc());

      // Get System Properties ...

      System.out.println("\nSystem Properties:");
      System.out.println("Operating System .................. " + System.getProperty("os.name"));
      System.out.println("Operating System Version .......... " + System.getProperty("os.version"));
      System.out.println("Language Code ..................... " + System.getProperty("user.language"));
      System.out.println("Country Code ...................... " + System.getProperty("user.region"));
      System.out.println("Time Zone ......................... " + System.getProperty("user.timezone"));

      System.out.println("\nJava Properties:");
      System.out.println("JVM locale ........................ " + System.getProperty("file.encoding"));
      System.out.println("Local encodings to Unicode package. " + System.getProperty("file.encoding.pkg"));
      System.out.println("Java Interpreter .................. " + System.getProperty("java.version"));
      System.out.println("Java Installation Directory ....... " + System.getProperty("java.home"));
      System.out.println("JIT Compiler ...................... " + System.getProperty("java.compiler"));
      System.out.println("Java Classpath .................... " + System.getProperty("java.class.path"));


      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection and exit
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      // manually pass iterations
      reportResults.reportEXIresults("cpgj001", 1, 1);
   }
}
