//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//3     exijs01 - JGate Open/Close Test for Memory Leak Detection
//
//p     This program is useful for detecting memory leaks where the
//      application is running. As this is a Java application the JGate
//      specified will generally be local for the results to be meaningful.
//
//p     There are three main options:
//o
//l         Issue continuous open/close requests of the Java Gateway
//l         Issue continuous ECI_listSystems requests between opening and closing
//                the Java Gateway
//l         Issue continuous EPI_listSystems requests between opening and closing
//                the Java Gateway
//
//p     Parameters can be overridden via command line.
//
//p     Issue "ecijlst -?" for help and default settings.
//
//p     Sample Input
//
//x     java jgtest.gateway.exijs01 -j local: -i 1000 -eci
//
//p     The sample input requests 1000 iterations of opening and closing
//      the Java Gateway, issuing an ECI_listSystems request. The number
//      of systems returned by the ECI_listSystems request is shown in
//      brackets in the last column of the output.
//
//p     Sample Output
//
//x     exijs01 - JGate Open/Close Test for Memory Leak Detection
//             -j     JGate              -> local:
//             -r     JGate port         -> 2006
//             -list  list systems only  -> false
//             -i     # iterations       -> 1000
//             -eci   ECI call           -> true
//             -epi   EPI call           -> false
//             -d     delay              -> 0 seconds
//             -!     debug              -> false
//             -?     for help panel
//
//      --- exijs01 started 18/06/98 15:23
//
//      Total       Free                                            ECI/EPI Rc +
//      Memory      Memory      Lowest      Highest     Iteration   (#Systems)
//      ----------- ----------- ----------- ----------- ----------- ------------
//      3145720     2761234     70736       3117400     1000        0 (5)
//
//
//      --- exijs01 completed 18/06/98 15:25
//
//      --- Iterations run ..... 1000 out of 1000
//
//
//p     Note: if output values for memory appear longer than 11 digits in
//            length, see code below marked "If Loads of Memory ..."
//p
//      History log:
//u
//l     Written : Jun 97 - Alan Hollingshead
//l     Updated : Apr 98 - uses common classes
//

package jgtest.gateway;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class exijs01
{
   public static void main(String[] args)
   {
      long hMem           = 0;
      long lMem           = 9999999999999999;
      int iTotalSystems   = 0;
      int iIt_Count       = 0;
      int    iRc          = 0;
      String sRc          = "n/a";
      String sSpace       = "                     ";

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nexijs01 - JGate Open/Close Test for Memory Leak Detection") ;
      System.out.println("       -j     JGate              -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port         -> "+theseParms.sPort);
      System.out.println("       -list  list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -i     # iterations       -> "+theseParms.sIterations);
      System.out.println("       -eci   ECI call           -> "+theseParms.bCallECI);
      System.out.println("       -epi   EPI call           -> "+theseParms.bCallEPI);
      System.out.println("       -d     delay              -> "+theseParms.sDelay+" seconds");
      System.out.println("       -!     debug              -> "+theseParms.bDebug);
      System.out.println("       -?     for help panel");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java exijs01 <parameters>");
         System.out.println(  "  e.g. java exijs01 -j local: -eci");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // ----------------------------------------------------------------------
         // Create connection to JGate
         // ----------------------------------------------------------------------

         JavaGateway jgaConnection = null;
         jgaConnection = setJGate.createJGate(theseParms);

         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("exijs01");

      // ----------------------------------------------------------------------
      // Display banner
      // ----------------------------------------------------------------------
      System.out.println("\nTotal       Free                                            ECI/EPI Rc +");
      System.out.println(  "Memory      Memory      Lowest      Highest     Iteration   (#Systems)");
      System.out.println(  "----------- ----------- ----------- ----------- ----------- ------------");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {

         // ----------------------------------------------------------------------
         // Create connection to JGate
         // ----------------------------------------------------------------------

         JavaGateway jgaConnection = null;
         try
         {
            jgaConnection = new JavaGateway(theseParms.sJGate, theseParms.iPort);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to create JavaGateway connection ---");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         if (theseParms.bCallECI)
         {
            // ---------------------------------------------------------------------
            // Issue ECIRequest.listSystems request
            // This does not require a server to be attached.
            // ---------------------------------------------------------------------
            ECIRequest eciRequest = null;
            eciRequest = ECIRequest.listSystems(99);
            try {
               jgaConnection.flow(eciRequest);
               iTotalSystems = (eciRequest.SystemList.size()/2);
               iRc = eciRequest.getRc();
               sRc = String.valueOf(iRc);
            }
            catch (Exception e)
            {
               System.out.println("\n--- ECIRequest.listSystems request failed ---");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }
         if (theseParms.bCallEPI) {
            // ---------------------------------------------------------------------
            // Issue EPIRequest.listSystems request
            // This does not require a server to be attached.
            // ---------------------------------------------------------------------
            EPIRequest epiRequest = null;
            epiRequest = EPIRequest.listSystems(99);
            try {
               jgaConnection.flow(epiRequest);
               iTotalSystems = (epiRequest.SystemList.size()/2);
               iRc = epiRequest.getRc();
               sRc = String.valueOf(iRc);
            }
            catch (Exception e)
            {
               System.out.println("\n--- EPIRequest.listSystems request failed ---");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }
         // ----------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection
         // ----------------------------------------------------------------------

         try
         {
            jgaConnection.close();
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error closing JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // ----------------------------------------------------------------------
         // Check memory situation
         // ----------------------------------------------------------------------
         long fMem = Runtime.getRuntime().freeMemory();
         long tMem = Runtime.getRuntime().totalMemory();

         // check for highest free memory value
         if ( fMem > hMem ) hMem = fMem;
         // check for lowest  free memory value
         if ( fMem < lMem ) lMem = fMem;


         // "If Loads of Memory ..."
         // Change '13' in next five lines to a larger value if necessary.
         // See note at top of program.
         String sGap1 = sSpace.substring(1,(13-(String.valueOf(tMem).length())));
         String sGap2 = sSpace.substring(1,(13-(String.valueOf(fMem).length())));
         String sGap3 = sSpace.substring(1,(13-(String.valueOf(lMem).length())));
         String sGap4 = sSpace.substring(1,(13-(String.valueOf(hMem).length())));
         String sGap5 = sSpace.substring(1,(13-(String.valueOf(iIt_Count).length())));
         //
         // -- end of "If Loads of Memory ..." ---

         System.out.print(tMem + sGap1 + fMem + sGap2 + lMem + sGap3
                                         + hMem + sGap4 + iIt_Count + sGap5
                                         + sRc  +" ("+iTotalSystems+")" +"  \r");

         System.out.flush();

         // Delay next iteration if delay parameter specified
         try {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e) {
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

      // end iIt_Count for-loop
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      System.out.println("\n");
      iIt_Count--;
      reportResults.reportEXIresults("exijs01",
                                     iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
