//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//3     gwyj001 - Local, Auto and Networked Java Gateways
//
//p     This programs tests:
//o
//l              Networked JavaGateway
//l              Local JavaGateway
//l              Auto JavaGateway
//l              LocalJavaGateway.destroy() method
//
//p     The order of tests is as follows:
//o
//l      Check NETWORKED JavaGateway is available, ECI Sync to NETWORKED, then close
//l      Create LOCAL JavaGateway and ECI Sync to LOCAL
//l      Re-open NETWORKED (TCP/IP) JavaGateway, then ECI Sync to NETWORKED
//l      Create AUTO (should default to Local:) JavaGateway, then ECI Sync to AUTO
//l      Explicitly close AUTO connection
//l      Call LocalJavaGateway.destroy() method - this should FAIL
//l      Explicitly close LOCAL connection
//l      Call to LocalJavaGateway.destroy() method
//l      Create LOCAL JavaGateway - should FAIL
//l      Create AUTO JavaGateway  - should FAIL
//l      ECI Sync to NETWORKED
//l      Explicitly close NETWORKED connection
//
//p     Parameters can be overridden via command line.
//
//p     Issue "gwyj001 -?" for help and default settings.
//
//p     Sample Input
//
//x     java jgtest.gateway.gwyj001 -j hollings
//
//p     The -j option must specify an active Java Gateway and should not be local:
//
//p     Sample Output
//
//x     gwyj001 - Local, Auto and Networked Java Gateways
//             -j     Networked JGate    -> hollings
//             -r     JGate port         -> 2006
//             -list  list systems only  -> false
//             -s     CICS server        -> (not specified)
//             -a     application        -> SRVTIME
//             -u     userid             -> SYSAD
//             -p     password           -> SYSAD
//             -!     debug              -> false
//             -?     Display settings only
//
//      --- gwyj001 started 18/06/98 16:54
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      1.  NETWORKED JavaGateway connection available
//          getAddress() ............ hollings
//          getGatewayLocale()....... en_GB
//          getPort() ............... 2006
//          getProtocol()............ tcp
//          getURL() ................ tcp://hollings:2006/
//          isInitialFlow() ......... true
//          isOpen() ................ true
//          eci_sync - commarea ..... 18/06/98 16:54:13 RC: 0
//          NETWORKED JavaGateway connection closed
//
//      2.  LOCAL JavaGateway connection available
//          via setURL(local:)
//          getAddress() ............
//          getGatewayLocale()....... null
//          getPort() ............... 0
//          getProtocol()............ local
//          getURL() ................ local:///
//          isInitialFlow() ......... true
//          isOpen() ................ false
//          LOCAL JavaGateway connection opened
//          isOpen() ................ true
//          eci_sync - commarea ..... 18/06/98 16:54:14 RC: 0
//
//      3.  NETWORKED JavaGateway connection re-opened
//          getAddress() ............ hollings
//          getGatewayLocale()....... en_GB
//          getPort() ............... 2006
//          getProtocol()............ tcp
//          getURL() ................ tcp://hollings:2006/
//          isInitialFlow() ......... true
//          isOpen() ................ true
//          eci_sync - commarea ..... 18/06/98 16:54:14 RC: 0
//
//      4.  AUTO JavaGateway connection available
//          getAddress() ............ hollings
//          getGatewayLocale()....... null
//          getPort() ............... 2006
//          getProtocol()............ auto
//          getURL() ................ auto://hollings:2006/
//          isInitialFlow() ......... true
//          isOpen() ................ true
//          AutoJavaGateway protocol. tcp
//
//          eci_sync - commarea ..... 18/06/98 16:54:15 RC: 0
//
//      5.  AUTO JavaGateway connection closed
//          isOpen() ................ false
//
//      6.  Call to LocalJavaGateway.destroy() raised expected exception.
//          Exception: java.io.IOException: CCL6659E: In use Local Gateways currently exist.
//
//      7.  LOCAL JavaGateway connection
//          isOpen() ................ true
//          LOCAL JavaGateway connection closed
//          isOpen() ................ false
//
//      8.  Call to LocalJavaGateway.destroy() successful
//
//      9.  Creation of LocalJavaGateway failed and raised expected exception.
//          Exception: java.io.IOException: CCL6658E: Local Gateway support has been terminated.
//
//      10. Creation of AutoJavaGateway failed and raised expected exception.
//          Exception: java.io.IOException: CCL6658E: Local Gateway support has been terminated.
//
//      11. Continue with NETWORKED JavaGateway connection
//          isOpen() ................ true
//          eci_sync - commarea ..... 18/06/98 16:54:15 RC: 0
//
//      12. Close NETWORKED JavaGateway connection
//          NETWORKED JavaGateway connection closed
//          isOpen() ................ false
//
//      --- gwyj001 completed 18/06/98 16:54
//
//      --- Calls successful ... 5 out of 5
//      --- Iterations run ..... 1 out of 1
//
//      --- gwyj001 : JGate steps completed successfully ... 12 out of 12
//
//p     History log:
//u
//l     Written : Nov 97 - Alan Hollingshead (based on destroyLocalGateway by Chris Goodall)
//l     Updated : Apr 98 - uses common classes
//

package jgtest.gateway;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class gwyj001
{
   static int     iCounter     = 0;
   static int     iReplyOK     = 0;
   static int     iCallsIssued = 0;
   static int     iTest        = 0;
   static int     iNumTests    = 12;  // issue gwyj001 -? for details


   public static void main(String[] args)
   {
      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\ngwyj001 - Local, Auto and Networked Java Gateways");
      System.out.println("       -j     Networked JGate    -> "+theseParms.sJGate);
      System.out.println("       -r     JGate port         -> "+theseParms.sPort);
      System.out.println("       -list  list systems only  -> "+theseParms.bListSystems);
      System.out.println("       -s     CICS server        -> "+theseParms.sServer);
      System.out.println("       -a     application        -> "+theseParms.sProgname);
      System.out.println("       -u     userid             -> "+theseParms.sUserid);
      System.out.println("       -p     password           -> "+theseParms.sPassword);
      System.out.println("       -!     debug              -> "+theseParms.bDebug);
      System.out.println("       -?     Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java gwyj001 <parameters>");
         System.out.println(  "  e.g. java gwyj001 -j hollings");
         System.out.println("\n-----------------------------------------------------------------------");
         System.out.println(" Testing steps:                                                        ");
         System.out.println("-----------------------------------------------------------------------");
         System.out.println("                                                                       ");
         System.out.println("  1. Check NETWORKED JavaGateway is available, then close after        ");
         System.out.println("     ECI Sync to NETWORKED                                             ");
         System.out.println("  2. Create LOCAL JavaGateway                                          ");
         System.out.println("     ECI Sync to LOCAL                                                 ");
         System.out.println("  3. Re-open NETWORKED (TCP/IP) JavaGateway                            ");
         System.out.println("     ECI Sync to NETWORKED JavaGateway                                 ");
         System.out.println("  4. Create AUTO (should default to Local:) JavaGateway                ");
         System.out.println("     ECI Sync to AUTO                                                  ");
         System.out.println("  5. Explicitly close AUTO connection                                  ");
         System.out.println("  6. Call LocalJavaGateway.destroy() method - this should FAIL         ");
         System.out.println("  7. Explicitly close LOCAL connection                                 ");
         System.out.println("  8. Call to LocalJavaGateway.destroy() method                         ");
         System.out.println("  9. Create LOCAL JavaGateway - should FAIL                            ");
         System.out.println(" 10. Create AUTO JavaGateway  - should FAIL                            ");
         System.out.println(" 11. ECI Sync to NETWORKED JavaGateway                                 ");
         System.out.println(" 12. Explicitly close NETWORKED connection                             ");
         System.out.println("                                                                       ");
         System.out.println("-----------------------------------------------------------------------\n");

         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("gwyj001");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection and exit
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ----------------------------------------------------------------------
      // Create gateway objects
      // ----------------------------------------------------------------------

      // create local connection object
      JavaGateway localConnection;

      // create network connection object
      JavaGateway networkConnection;

      // create auto connection object
      JavaGateway autoConnection;

      // use test_loop to break out if unexpected exception caught. Then
      // report the results. The loop is only executed once.

      test_loop:
      for (int iTestLoop=1; iTestLoop<=1; iTestLoop++)
      {
         // ----------------------------------------------------------------------
         //  1. Check NETWORKED JavaGateway is available, then close
         // ----------------------------------------------------------------------

         // create networked connection
         networkConnection=null;
         try
         {
            networkConnection = new JavaGateway("tcp://"+theseParms.sJGate, theseParms.iPort);
            iTest++;
            System.out.println("\n" + iTest +".  NETWORKED JavaGateway connection available");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to create NETWORKED JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    getAddress() ............ " + networkConnection.getAddress() );
         System.out.println("    getGatewayLocale()....... " + networkConnection.getGatewayLocale() );
         System.out.println("    getPort() ............... " + networkConnection.getPort() );
         System.out.println("    getProtocol()............ " + networkConnection.getProtocol() );
         System.out.println("    getURL() ................ " + networkConnection.getURL() );
         System.out.println("    isInitialFlow() ......... " + networkConnection.isInitialFlow() );
         System.out.println("    isOpen() ................ " + networkConnection.isOpen() );

         // issue eci_sync call
         eciSync(networkConnection, theseParms);

         // close NETWORKED connection
         try
         {
            networkConnection.close();
            System.out.println("    NETWORKED JavaGateway connection closed");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error closing NETWORKED JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         // ----------------------------------------------------------------------
         //  2. Create LOCAL JavaGateway
         // ----------------------------------------------------------------------

         localConnection=null;

         try
         {
            localConnection = new JavaGateway();
            localConnection.setURL("local:");
            iTest++;
            System.out.println("\n" + iTest +".  LOCAL JavaGateway connection available");
            System.out.println("    via setURL(local:)");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to create LOCAL JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    getAddress() ............ " + localConnection.getAddress() );
         System.out.println("    getGatewayLocale()....... " + localConnection.getGatewayLocale() );
         System.out.println("    getPort() ............... " + localConnection.getPort() );
         System.out.println("    getProtocol()............ " + localConnection.getProtocol() );
         System.out.println("    getURL() ................ " + localConnection.getURL() );
         System.out.println("    isInitialFlow() ......... " + localConnection.isInitialFlow() );
         System.out.println("    isOpen() ................ " + localConnection.isOpen() );

         try
         {
            localConnection.open();
            System.out.println("    LOCAL JavaGateway connection opened");
            System.out.println("    isOpen() ................ " + localConnection.isOpen() );
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to open LOCAL JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         // issue eci_sync call
         eciSync(localConnection, theseParms);

         // ----------------------------------------------------------------------
         //  3. Re-open NETWORKED JavaGateway
         // ----------------------------------------------------------------------

         try
         {
            networkConnection.open();
            iTest++;
            System.out.println("\n" + iTest +".  NETWORKED JavaGateway connection re-opened");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to re-open NETWORKED JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    getAddress() ............ " + networkConnection.getAddress() );
         System.out.println("    getGatewayLocale()....... " + networkConnection.getGatewayLocale() );
         System.out.println("    getPort() ............... " + networkConnection.getPort() );
         System.out.println("    getProtocol()............ " + networkConnection.getProtocol() );
         System.out.println("    getURL() ................ " + networkConnection.getURL() );
         System.out.println("    isInitialFlow() ......... " + networkConnection.isInitialFlow() );
         System.out.println("    isOpen() ................ " + networkConnection.isOpen() );

         // issue eci_sync call
         eciSync(networkConnection, theseParms);

         // ----------------------------------------------------------------------
         //  4. Create AUTO JavaGateway
         // ----------------------------------------------------------------------

         autoConnection=null;

         try
         {
            autoConnection = new JavaGateway("auto://"+theseParms.sJGate, theseParms.iPort);
            iTest++;
            System.out.println("\n" + iTest +".  AUTO JavaGateway connection available");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to create AUTO JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    getAddress() ............ " + autoConnection.getAddress() );
         System.out.println("    getGatewayLocale()....... " + autoConnection.getGatewayLocale() );
         System.out.println("    getPort() ............... " + autoConnection.getPort() );
         System.out.println("    getProtocol()............ " + autoConnection.getProtocol() );
         System.out.println("    getURL() ................ " + autoConnection.getURL() );
         System.out.println("    isInitialFlow() ......... " + autoConnection.isInitialFlow() );
         System.out.println("    isOpen() ................ " + autoConnection.isOpen() );
         System.out.println("    AutoJavaGateway protocol. " + AutoJavaGateway.getNetworkProtocol() );

    //   System.out.println("\n ** How to prove using local other than seeing trace?");

         // issue eci_sync call
         eciSync(autoConnection, theseParms);

         // ----------------------------------------------------------------------
         //  5. Close AUTO JavaGateway
         // ----------------------------------------------------------------------

         try
         {
            autoConnection.close();
            iTest++;
            System.out.println("\n" + iTest +".  AUTO JavaGateway connection closed");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to close AUTO JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    isOpen() ................ " + autoConnection.isOpen() );

         // ----------------------------------------------------------------------
         //  6. Call LocalJavaGateway.destroy() method - this should FAIL
         // ----------------------------------------------------------------------

         try
         {
            LocalJavaGateway.destroy();
            System.out.println("\nERROR - Call to LocalJavaGateway.destroy() was successful.");
            System.out.println("One LocalJavaGateway still open, so this should have raised an exception.");
            break test_loop;;
         }
         catch (Exception e)
         {
            T.ln(null,"OK: Caught expected exception: ");
            T.ex(null,e);
            iTest++;
            System.out.println("\n" + iTest + ".  Call to LocalJavaGateway.destroy() raised expected exception.");
            System.out.println("    Exception: " + e);
         }

         // ----------------------------------------------------------------------
         //  7. Close LOCAL JavaGateway
         // ----------------------------------------------------------------------

         iTest++;
         System.out.println("\n" + iTest +".  LOCAL JavaGateway connection");
         System.out.println("    isOpen() ................ " + localConnection.isOpen() );

         try
         {
            localConnection.close();
            System.out.println("    LOCAL JavaGateway connection closed");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Failed to close LOCAL JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    isOpen() ................ " + localConnection.isOpen() );

         // ----------------------------------------------------------------------
         //  8. Call LocalJavaGateway.destroy() method - this should be successful
         // ----------------------------------------------------------------------

         try
         {
            iTest++;
            LocalJavaGateway.destroy();
            System.out.println("\n" + iTest + ".  Call to LocalJavaGateway.destroy() successful");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\nERROR - Call to LocalJavaGateway.destroy() failed");
            System.out.println("Exception: " + e);
            break test_loop;;
         }

         // ----------------------------------------------------------------------
         //  9. Try to create another LOCAL JavaGateway - this should fail
         // ----------------------------------------------------------------------


         JavaGateway anotherLocalConnection;
         anotherLocalConnection=null;

         try
         {
            anotherLocalConnection = new JavaGateway("local:",0);
            System.out.println("\nERROR - Creation of LocalJavaGateway was successful.");
            System.out.println("LocalJavaGateway.destroy() previously issued.");
            break test_loop;;
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            iTest++;
            System.out.println("\n" + iTest + ".  Creation of LocalJavaGateway failed and raised expected exception.");
            System.out.println("    Exception: " + e);
         }

         // ----------------------------------------------------------------------
         //  10. Try to create another AUTO JavaGateway - this should fail
         // ----------------------------------------------------------------------

         // The AUTO type of gateway will attempt to create a connection using the
         // local protocol - for this reason it should fail because we have
         // disabled LOCAL connections with LocalJavaGateway.destroy()

         JavaGateway anotherAutoConnection;
         anotherAutoConnection=null;

         try
         {
            anotherAutoConnection = new JavaGateway("auto://"+theseParms.sJGate, theseParms.iPort);
            System.out.println("\nERROR - Creation of AutoJavaGateway was successful.");
            System.out.println("LocalJavaGateway.destroy() previously issued.");
            break test_loop;;
         }
         catch (Exception e)
         {
            T.ln(null,"OK: Caught expected exception: ");
            T.ex(null,e);
            iTest++;
            System.out.println("\n" + iTest + ". Creation of AutoJavaGateway failed and raised expected exception.");
            System.out.println("    Exception: " + e);
         }


         // ----------------------------------------------------------------------
         // 11. ECI Sync to existing NETWORKED JavaGateway
         // ----------------------------------------------------------------------

         iTest++;
         System.out.println("\n" + iTest +". Continue with NETWORKED JavaGateway connection");
         System.out.println("    isOpen() ................ " + networkConnection.isOpen() );

         // issue eci_sync call
         eciSync(networkConnection, theseParms);

         // ----------------------------------------------------------------------
         // 12. Close NETWORKED JavaGateway
         // ----------------------------------------------------------------------

         iTest++;
         System.out.println("\n" + iTest +". Close NETWORKED JavaGateway connection");

         // close NETWORKED connection
         try
         {
            networkConnection.close();
            System.out.println("    NETWORKED JavaGateway connection closed");
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error closing NETWORKED JavaGateway connection ---\n");
            System.out.println("--- Caught Exception: " + e);
            break test_loop;;
         }

         System.out.println("    isOpen() ................ " + networkConnection.isOpen() );

      }  // end test_loop

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      // manually pass iterations
      reportResults.reportECIresults("gwyj001", iReplyOK, iCallsIssued, 1, 1);

      String sTestsOK = (": JGate steps completed successfully ... " +
                            iTest + " out of " + iNumTests);

      reportResults.reportComment("gwyj001", sTestsOK);

      System.exit(0);

   } // end-main() method

   // ------------------------------------------------------------------------
   // Method: eciSync()
   // Performs a synchronous ECI call
   // ------------------------------------------------------------------------
   public static boolean eciSync(JavaGateway currentConnection,
                                 inputParms theseParms )
   {
      boolean bECIReqOK = false;

      // CICS commarea
      byte[] Commarea = new byte[18];

      // ------------------------------------------------------------------
      // Use extended contructor ECIRequest() for synchronous call
      // ------------------------------------------------------------------

         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                                  theseParms.sServer,
                                                  theseParms.sUserid,
                                                  theseParms.sPassword,
                                                  theseParms.sProgname,
                                                  theseParms.sTransid,
                                                  Commarea,
                                                  0,
                                                  ECIRequest.ECI_NO_EXTEND,
                                                  ECIRequest.ECI_LUW_NEW );

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            String sDashData  = new String(eciRequest.Commarea,0);

         // Flow synchronous request
         try
         {
            iCallsIssued++;
            currentConnection.flow(eciRequest);
            bECIReqOK = true;
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_sync flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            bECIReqOK = false;
         }

         // Convert ASCII bytes to ASCII characters
         String ComData  = new String(eciRequest.Commarea,0);

         System.out.println("    eci_sync - commarea ..... " +
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     " RC: " + eciRequest.Cics_Rc );

         // for debug purposes
         T.ln(null,"Back again : AbendCode = " +
                               eciRequest.Abend_Code);
         T.ln(null,"eci RC " + eciRequest.Cics_Rc);

         // keep tally of successful return of commarea
         if ((eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) &&
             (!ComData.equals("------------------"))  )
         {
            iReplyOK++;
         } // end-if

     // return whether the ECI-Request was a success or not
     return(bECIReqOK);
   }

} // end-gwyj001 class
