//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//3     ecijnls - Send and Check NLS Data
//
//p     This application sends NLS data found in a data file that is
//      also accessible by the CICS server (eg on a LAN drive) to check that the
//      data has been sent correctly. The data is sent using an ECI commarea.
//
//p     The fixed CICS server application SRVNLS reads the first four
//      characters of the incoming commarea. This contains the name of the
//      extrapartition TDQ that CICS (eg CICS for OS/2 v3) must have defined
//      pointing to the same LAN file as specified by the -file option
//      during the Java program invocation.
//
//p     The data received by the CICS server program in the commarea is
//      compared to the data read by CICS of the lan file (pointed to
//      by the extrapartition dataset). If the data matches, the call
//      is deemed successful.
//
//p     Parameters can be overridden via command line.
//
//p     Issue "ecijnls -?" for help and default settings.
//
//p     Sample Input
//x     java jgtest.nls.ecijnls -j local: -file T:\SVT\AXH\NLS\NLS-US.DAT
//
//p     Sample Output
//
//x     ecijnls - Send and Check NLS Data
//             -j      JGate                    -> local:
//             -r      JGate port               -> 2006
//             -list   list systems only        -> false
//             -s      CICS server              -> (not specified)
//             (fixed) application              -> SRVNLS
//             -file   data file                -> t:\svt\axh\nls\nls-us.dat
//             -i      # iterations             -> 1
//             -u      userid                   -> SYSAD
//             -p      password                 -> SYSAD
//             -d      delay between iterations -> 0 seconds
//             -x      exit on ECI error        -> false
//             -!      debug                    -> false
//             -?      Display settings only
//
//      --- ecijnls started 18/06/98 17:09
//
//      --- Successfully created JavaGateway connection ---
//
//      --- Using CICS Region: NETBTCLR
//
//      I/O: File t:\svt\axh\nls\nls-us.dat opened
//           .. reading in data ..
//      I/O: File t:\svt\axh\nls\nls-us.dat closed
//
//      #  Flow      Commarea-Data 1-30                Length CICS Rc  Abend
//      -- --------  ------------------------------    ------ -------  -----
//      1  --> Sent  NLUS Top Line - Do Not Change     40
//      1  <-- Rcvd  TDQ found                         40     0        null
//      1  --> Sent  NLUS First line of NLS data       27
//      1  <-- Rcvd  Data matched!                     27     0        null
//      1  --> Sent  NLUS Line2                        10
//      1  <-- Rcvd  Data match                        10     0        null
//      1  --> Sent  NLUS Third line                   15
//      1  <-- Rcvd  Data matched!                     15     0        null
//      1  --> Sent  NLUS Fourth and final line        26
//      1  <-- Rcvd  Data matched!                     26     0        null
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- ecijnls completed 18/06/98 17:09
//
//      --- Calls successful ... 5 out of 5
//      --- Iterations run ..... 1 out of 1
//
//
//p
//      History log:
//u
//l     Written : Apr 98 - Alan Hollingshead
//
package jgtest.nls;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

import java.io.*;
import java.util.*;

public class ecijnls
{
   public static void main(String[] args)
   {
      int     iIt_Count      = 0;
      int     iCounter       = 0;
      int     iCallsIssued   = 0;
      int     iReplyOK       = 0;
      boolean bCommUpdated   = false;

      // CICS commarea
      byte[] Commarea = new byte[30];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // set server application
      theseParms.sProgname = "SRVNLS";

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\necijnls - Send and Check NLS Data") ;
      System.out.println("       -j      JGate                    -> "+theseParms.sJGate);
      System.out.println("       -r      JGate port               -> "+theseParms.sPort);
      System.out.println("       -list   list systems only        -> "+theseParms.bListSystems);
      System.out.println("       -s      CICS server              -> "+theseParms.sServer);
      System.out.println("       (fixed) application              -> "+theseParms.sProgname);
      System.out.println("       -file   data file                -> "+theseParms.sFilename);
      System.out.println("       -i      # iterations             -> "+theseParms.sIterations);
      System.out.println("       -u      userid                   -> "+theseParms.sUserid);
      System.out.println("       -p      password                 -> "+theseParms.sPassword);
      System.out.println("       -d      delay between iterations -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x      exit on ECI error        -> "+theseParms.bExitNow);
      System.out.println("       -!      debug                    -> "+theseParms.bDebug);
      System.out.println("       -?      Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java ecijnls <parameters>");
         System.out.println(  "  e.g. java ecijnls -s AlsKix -file T:\\NLS\\NLS-US.DAT");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("ecijnls");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;
      jgaConnection = setJGate.createJGate(theseParms);

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ------------------------------------------------------------------
      // Read data from file into vector
      // ------------------------------------------------------------------

      String sLine = null;
      FileInputStream file = null;
      InputStreamReader isReader = null;
      BufferedReader brReader = null;

      Vector vFileData = new Vector();
      Enumeration enum = null;

      try
      {
         file = new FileInputStream(theseParms.sFilename);
         isReader = new InputStreamReader(file);
         brReader = new BufferedReader(isReader);

         System.out.println("\nI/O: File " + theseParms.sFilename + " opened");
         sLine = brReader.readLine();
         System.out.println("     .. reading in data ..");
         while (sLine != null)
         {
            vFileData.addElement(sLine);
            sLine = brReader.readLine();
         }
      }
      catch (FileNotFoundException fnfe)
      {
            System.err.println("Error: "+ fnfe);
            System.exit(0);
      }
      catch (IOException ioe) {
            System.err.println("Error writing output: "+ ioe);
            System.exit(0);
      }
      finally
      {
         try
         {
            file.close();
            System.out.println("I/O: File " + theseParms.sFilename + " closed");
         }
         catch (IOException ioe)
         {
            System.err.println("Error closing file: "+ ioe);
            System.exit(0);
         }
      }

      // ------------------------------------------------------------------
      // Use extended contructor ECIRequest() for synchronous call
      // ------------------------------------------------------------------

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++) {

         // display banner every iteration
         System.out.println("\n#  Flow      Commarea-Data 1-30                Length CICS Rc  Abend");
         System.out.println(  "-- --------  ------------------------------    ------ -------  -----");

         for ( enum = vFileData.elements(); enum.hasMoreElements(); ) {

            String sData = (String) enum.nextElement();
            Commarea=sData.getBytes();

            ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                                     theseParms.sServer,
                                                     theseParms.sUserid,
                                                     theseParms.sPassword,
                                                     theseParms.sProgname,
                                                     theseParms.sTransid,
                                                     Commarea,
                                                     0,
                                                     ECIRequest.ECI_NO_EXTEND,
                                                     ECIRequest.ECI_LUW_NEW );

            String sOriginalData = sData;

            if (sData.length() > 30)
               sData = sData.substring(0,30);
            String sSpace33  = "                                 ";
            String sGap1 = sSpace33.substring(1,(32-(String.valueOf(sData).length())));
            System.out.println(iIt_Count+"  --> Sent  " +
                    sData + sGap1 +
                    "   " + eciRequest.Commarea_Length );

            // Flow synchronous request
            try
            {
               iCallsIssued++;
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e)
            {
               T.ln(null,"ERROR: Caught Exception: ");
               T.ex(null,e);
               System.out.println("\n--- Error during eci_sync flow ---\n");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }

            // Convert ASCII bytes to ASCII characters
            String ComData  = new String(eciRequest.Commarea);

            if (ComData.equals(sOriginalData))
            {
               ComData      = "Commarea not updated";
               bCommUpdated = false;
            }
            else
            {
               bCommUpdated = true;
            }

            if (ComData.length() > 30)
            {
               ComData = ComData.substring(0,30);
            }

            String sSpace34  = "                                   ";
            String sGap2 = sSpace34.substring(1,(33-(ComData.length())));
            String sSpace6  = "       ";
            String sGap3 = sSpace6.substring(1,(5-(String.valueOf(eciRequest.Commarea_Length).length())));
            System.out.println(iIt_Count+"  <-- Rcvd  " +
                        ComData.substring(0) + "  " +sGap2+
                        eciRequest.Commarea_Length +"   " +sGap3 +
                        eciRequest.Cics_Rc + "        " +
                        eciRequest.Abend_Code );

            // keep tally of successful return of commarea
            if (   (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
                && (bCommUpdated) )
            {
               iReplyOK++;
            } // end-if

         } // end-for-iString_Count

         // Delay next ECI call if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("ecijnls", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
