//Ŀ
//                                                                       
//  MODULE NAME      CA88.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME CICS Gateway for Java v2 Test Suite                 
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA88 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
//p     This application contains three very simple calls (identical to
//      ecij123) with user-specified security classes.
//
//p     Parameters can be overridden via command line.
//
//p     Issue "securityTest -?" for help and default settings.
//
//p     Sample Input
//
//x     java securityTest -s AirsKix -clisec com.ibm.ctg.security.clientDesign
//                        -srvsec com.ibm.ctg.security.serverDesign
//
//p     The above example uses the AirsKix server, and the JGate's sample
//      encryption classes, clientDesign and serverDesign.
//
//p     Sample Output
//
//x     securityTest - Three Simple ECI Calls with Security
//             -j      JGate                    -> local:
//             -r      JGate port               -> 2006
//             -list   list systems only        -> false
//             -s      CICS server              -> (not specified)
//             -clisec Client Security          -> com.ibm.ctg.security.clientDesign
//             -srvsec Server Security          -> com.ibm.ctg.security.serverDesign
//             -a      application              -> SRVTIME
//             -i      # iterations             -> 1
//             -u      userid                   -> SYSAD
//             -p      password                 -> SYSAD
//             -d      delay between iterations -> 0 seconds
//             -x      exit on ECI error        -> false
//             -!      debug                    -> false
//             -?      Display settings only
//
//      --- securityTest started 18/06/98 11:51
//
//      --- creating secure connection...
//
//      --- Successfully created Secure JavaGateway connection ---
//
//      --- Using CICS Region: TCLRTST1
//
//      #  Call               Commarea-Data      Length CICS Rc  Abend
//      -- ------------------ -----------------  ------ -------  -----
//      1  eci_sync           18/06/98 11:50:48  18     0        null
//      1  eci_async          -----------------  18     0        null
//      1  eci_get_reply_wait 18/06/98 11:50:49  18     0        null
//
//
//      --- Successfully closed JavaGateway connection ---
//
//      --- securityTest completed 18/06/98 11:51
//
//      --- Calls successful ... 3 out of 3
//      --- Iterations run ..... 1 out of 1
//
//p     History Log:
//u
//l     Written : Apr 97 - Alan Hollingshead (ecij123)
//l     Updated : Nov 97 - updated to test SecurityExits - JGate v2.0 (Chris Goodall)
//l     Updated : Jun 98 - uses inputParms, ECIlist, setJGate, + other classes

package jgtest.security;

import ibm.cics.jgate.client.*;
import jgtest.util.*;

public class securityTest
{
   public static void main(String[] args)
   {
      int     iEci_Call      = 0;
      int     iLUW           = 0;
      int     iIt_Count      = 0;
      int     iCounter       = 0;
      int     iCallsIssued   = 0;
      int     iReply_Count   = 0;
      int     iReplyOK       = 0;
      String  sGetReply      = "";
      boolean bReply_Rcvd    = false;

      // CICS commarea
      byte[] Commarea = new byte[18];

      // ----------------------------------------------------------------------
      // Call class inputParms to check input parameters and set defaults
      // ----------------------------------------------------------------------

      inputParms theseParms;
      theseParms = new inputParms(args);

      // ----------------------------------------------------------------------
      // Display settings
      // ----------------------------------------------------------------------

      System.out.println("\nsecurityTest - Three Simple ECI Calls with Security") ;
      System.out.println("       -j      JGate                    -> "+theseParms.sJGate);
      System.out.println("       -r      JGate port               -> "+theseParms.sPort);
      System.out.println("       -list   list systems only        -> "+theseParms.bListSystems);
      System.out.println("       -s      CICS server              -> "+theseParms.sServer);
      System.out.println("       -clisec Client Security          -> "+theseParms.sClientSecurity);
      System.out.println("       -srvsec Server Security          -> "+theseParms.sServerSecurity);
      System.out.println("       -a      application              -> "+theseParms.sProgname);
      System.out.println("       -i      # iterations             -> "+theseParms.sIterations);
      System.out.println("       -u      userid                   -> "+theseParms.sUserid);
      System.out.println("       -p      password                 -> "+theseParms.sPassword);
      System.out.println("       -d      delay between iterations -> "+theseParms.sDelay+" seconds");
      System.out.println("       -x      exit on ECI error        -> "+theseParms.bExitNow);
      System.out.println("       -!      debug                    -> "+theseParms.bDebug);
      System.out.println("       -?      Display settings only");

      // ----------------------------------------------------------------------
      // Display help if requested, then terminate
      // ----------------------------------------------------------------------

      if (theseParms.bHelp)
      {
         System.out.println("\nUsage: java securityTest <parameters>");
         System.out.println(  "  e.g. java securityTest -s AirKix -clisec com.ibm.ctg.security.clientDesign");
         System.out.println(  "                         -srvsec com.ibm.ctg.security.serverDesign");
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // Call class reportResults to display start time
      // ----------------------------------------------------------------------

      reportResults.reportStarted("securityTest");

      // ----------------------------------------------------------------------
      // Create connection to JGate
      // ----------------------------------------------------------------------

      JavaGateway jgaConnection = null;

      if (theseParms.sClientSecurity.equals("(not specified)") ||
          theseParms.sServerSecurity.equals("(not specified)") ) {
        System.out.println("\n--- connecting with NO SECURITY: client and/or server classes not specified");
        jgaConnection = setJGate.createJGate(theseParms);
      }
      else {
        System.out.println("\n--- creating secure connection...");
        jgaConnection = setJGate.createSecureJGate(theseParms);
      }

      // ----------------------------------------------------------------------
      // Call class ECIlist if list systems required
      // ----------------------------------------------------------------------

      if (theseParms.bListSystems)
      {
         // call class ECIlist to list systems available
         ECIlist.allSystems(jgaConnection);

         // -------------------------------------------------------------------
         // Call class closeJGate to close Java Gateway connection
         // -------------------------------------------------------------------

         setJGate.closeJGate(jgaConnection);
         System.exit(0);
      }

      // ----------------------------------------------------------------------
      // If server name has not been provided, call getServer1 method
      // ----------------------------------------------------------------------

      if (theseParms.sServer.equals("(not specified)"))
      {
         theseParms.sServer = ECIlist.getServer1(jgaConnection);
      }

      // ------------------------------------------------------------------
      // Use extended contructor ECIRequest() for synchronous call
      // ------------------------------------------------------------------

      System.out.println("\n#  Call               Commarea-Data      Length CICS Rc  Abend");
      System.out.println("-- ------------------ -----------------  ------ -------  -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=theseParms.iIterations; iIt_Count++)
      {
         // output banner every 10 iterations
         if (iIt_Count%10==0)
         {
            System.out.println("\n#   Call               Commarea-Data      Length CICS Rc  Abend");
            System.out.println("--  ------------------ -----------------  ------ -------  -----");
         }

         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                                  theseParms.sServer,
                                                  theseParms.sUserid,
                                                  theseParms.sPassword,
                                                  theseParms.sProgname,
                                                  theseParms.sTransid,
                                                  Commarea,
                                                  0,
                                                  ECIRequest.ECI_NO_EXTEND,
                                                  ECIRequest.ECI_LUW_NEW );

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         String sDashData  = new String(eciRequest.Commarea);

         // Flow synchronous request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_sync flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         // ,0 required for OS/390
         String ComData  = new String(eciRequest.Commarea,0);

         System.out.println(iIt_Count+"  eci_sync           " +
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // ------------------------------------------------------------------
         // ECI_ASYNC
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_ASYNC;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            sDashData = new String(eciRequest.Commarea);

         // flow ECI request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_async flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea);

         // Display result
         System.out.println(iIt_Count+"  eci_async          "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code );

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT
         // ------------------------------------------------------------------

         eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;

         // reset commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;

         // Convert ASCII bytes to ASCII characters
         sDashData = new String(eciRequest.Commarea);

         // flow ECI request
         try
         {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during eci_get_reply_wait flow ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }

         // Convert ASCII bytes to ASCII characters
         ComData = new String(eciRequest.Commarea);

         // Display result
         System.out.println(iIt_Count+"  eci_get_reply_wait "+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc + "        " +
                     eciRequest.Abend_Code + "\n");

          // Check result
          if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
          {
             iReplyOK++;
          }
          else
          {
             if (theseParms.bExitNow)
             {
                System.out.println("\n--- ECI CICS_RC error - aborting program");
                break iteration_loop;
             }
          }

         // Delay next ECI call if delay parameter specified
         try
         {
            Thread.sleep(theseParms.iDelay);
         }
         catch (Exception e)
         {
            T.ln(null,"ERROR: Caught Exception: ");
            T.ex(null,e);
            System.out.println("\n--- Error during sleep() ---\n");
            System.out.println("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count

      // ----------------------------------------------------------------------
      // Call class closeJGate to close Java Gateway connection
      // ----------------------------------------------------------------------

      setJGate.closeJGate(jgaConnection);

      // ----------------------------------------------------------------------
      // Call class reportResults to display results
      // ----------------------------------------------------------------------

      reportResults.reportECIresults("securityTest", iReplyOK, iCallsIssued,
                                     --iIt_Count, theseParms.iIterations);

      System.exit(0);
   }
}
