       identification division.
       program-id. DLYTIME
       environment division.
       data division.
       working-storage section.
      **************************************************************
      * Simple server program that returns
      * date and time into the commarea
      * AFTER a delay of 'nnn' seconds as requested by the
      * client program in the incoming commarea.
      * Commarea received must be at least 18 characters long
      * and may be up to 65536 (64K) characters long.
      *
      * Author: Alan Hollingshead - 29 August 1997
      **************************************************************
       01 ws-dlytime-working-storage.
            05  filler            pic x(22) value
                                  'working storage begins'.
       01 resp                    pic s9(8) comp.
       01 abs-time                pic s9(15) comp-3.

       01 wto-msg                 pic x(80) value spaces.
       01 comm-len-received       pic 9(5).

       01 hang-about-c            pic x(3).
       01 hang-about              pic s9(8) comp.
       01 msg1                    pic x(80) value
              'commarea less than length 18. Cannot continue.'.

       linkage section.
         01 dfhcommarea.
            05 date-area        pic x(8).
            05 separator        pic x(1).
            05 time-area        pic x(8).
            05 lowval           pic x(1).
            05 rest-commarea    pic x
               occurs 19 to 32767 depending on eibcalen.

       procedure division.

       abc-main section.

      * check commarea has been sent
           if eibcalen equal 0
              string 'DLYTIME: commarea of '  delimited by size
                     '18+ characters must be' delimited by size
                     '  supplied. Resend '    delimited by size
                     'request.'               delimited by size
                     into wto-msg
              end-string
              perform send-msg

              exec cics return
              end-exec
              go to abc-exit.


      * check commarea sent is 18+ characters in length
           if eibcalen less than 18
              string 'DLYTIME: commarea '     delimited by size
                     'less than length 18.'   delimited by size
                     '  Resend request with'  delimited by size
                     ' longer commarea.'      delimited by size
                     into wto-msg
              end-string
              perform send-msg

              exec cics return
              end-exec
              go to abc-exit.

           move eibcalen to comm-len-received.

           move date-area(1:3) to hang-about-c.
           move hang-about-c to hang-about.

      * ask time before delay
           exec cics asktime abstime(abs-time)
           end-exec.

           exec cics formattime abstime(abs-time)
                                ddmmyy(date-area)
                                datesep('/')
                                time(time-area)
                                timesep(':')
           end-exec.

           move space to separator.
           move low-value to lowval.

           string 'DLYTIME: time before '  delimited by size
                  hang-about-c             delimited by size
                  ' second delay: '        delimited by size
                  date-area                delimited by size
                  ' '                      delimited by size
                  time-area                delimited by size
                  into wto-msg
           end-string
           perform send-msg.

      * delay one minute
           exec cics delay for seconds(hang-about)
                               reqid(eibtrmid)
           end-exec.

      * now ask the time again
           exec cics asktime abstime(abs-time)
           end-exec.

           exec cics formattime abstime(abs-time)
                                ddmmyy(date-area)
                                datesep('/')
                                time(time-area)
                                timesep(':')
           end-exec.

           move space to separator.
           move low-value to lowval.

           string 'DLYTIME: time after  '  delimited by size
                  hang-about-c             delimited by size
                  ' second delay: '        delimited by size
                  date-area                delimited by size
                  ' '                      delimited by size
                  time-area                delimited by size
                  into wto-msg
           end-string
           perform send-msg.

      *  now return

           exec cics return
           end-exec.

       abc-exit.
           exit.

      *--------------------------------------------------------*

       send-msg.
      *    Comment out as required. Options are to send output
      *    to CSMT, to console (if supported); both or neither.

      *    exec cics writeq td queue('CSMT') from(wto-msg)
      *              length(80)
      *              nohandle end-exec.

           exec cics write operator text(wto-msg)
                     textlength(length of wto-msg)
           end-exec.
       send-msg-exit.
           exit.

