       identification division.
       program-id. SRVNLS.
       environment division.
       data division.
       working-storage section.
      *-----------------------------------------------------------------
      * Requires: (1) JGate program ecijnls as F/E program.
      *           (2) Data file accessible by both Java application and
      *               this program (for example, on a shared LAN drive)
      *           (3) CICS TDQ (extrapartition pointing to data file
      *               described in (2)
      *           (4) This program to receive the data from ecijnls
      *               and compare it with the data on the LAN drive
      *               as read by CICS
      *
      * (1) ecijnls - issue "ecijnls -?" for usage and defaults
      * (2) Data file - each input line must begin with:
      *     "<TDQ name> <space> data"
      *     For example:
      *     "NLUS First line of data"
      * (3) This program reads the data file as pointed to by the
      *     TDQ passed in by the data file in (2)
      * (4) This program confirms the TDQ is accessible, reads the
      *     data into a TSQ and compares each line of data received
      *     with the data in the TSQ.
      *
      * Author: Alan Hollingshead - May 1998
      *-----------------------------------------------------------------
       01 ws-nlsfile-working-storage.
            05  filler            pic x(22) value
                                  'working storage begins'.
       01  ws-misc.
          02  ws-progname            pic x(6) value 'SRVNLS'.
          02  ws-open                pic s9(8) comp value 18.
          02  ws-closed              pic s9(8) comp value 19.
          02  ws-tdq                 pic x(4) value spaces.
          02  ws-tsq                 pic x(4) value spaces.
          02  ws-td-record           pic x(80) value spaces.
          02  ws-ts-record           pic x(80) value spaces.

          02  ws-cics-resp           pic 9(8) comp.
          02  ws-cics-resp-char      pic x(8).

          02  ws-send-text           pic x(80) value spaces.
          02  ws-comm-len-rcvd       pic 9(5).

       linkage section.
         01 dfhcommarea.
            05 cics-commarea    pic x
               occurs 1 to 32767 depending on eibcalen.

      *-----------------------------------------------------------------
       procedure division.
      *-----------------------------------------------------------------
       main section.
       a1.
           perform check-input-data.
           perform termination.
       a-end.
           exit.

      *-----------------------------------------------------------------
       check-input-data section.
       lt.
      * check commarea has been sent
           if eibcalen equal 0
              string ws-progname               delimited by size
                     ': commarea length zero.' delimited by size
                     'Resend request.'         delimited by size
                     into ws-send-text
              end-string
              perform send-msg
              perform termination
           end-if
           .

      * first four characters of each line used for TDQ and TSQ
           move dfhcommarea to ws-tdq ws-tsq

      * check if first time through by checking top line
            if dfhcommarea(6:8) = 'Top Line'
              exec cics deleteq ts queue(ws-tsq)
                        resp(ws-cics-resp)
              end-exec
            end-if

      * query if commarea data equals known TSQ
           exec cics inquire tsqueue(dfhcommarea(1:4))
                     resp(ws-cics-resp)
           end-exec

      * if TSQ does not exists
      * ---
      * --- first time through (start) ---
      * ---
           if ws-cics-resp = dfhresp(QIDERR) then

      *       string ws-progname              delimited by size
      *              ': TSQ '                 delimited by size
      *              ws-tsq                   delimited by size
      *              ' does not exist.'       delimited by size
      *              into ws-send-text
      *       end-string
      *       perform send-msg

      * confirm TDQ exists

              exec cics inquire tdqueue(ws-tdq)
                        resp(ws-cics-resp)
              end-exec

              if ws-cics-resp = dfhresp(QIDERR) then

                 string ws-progname              delimited by size
                        ': TDQ '                 delimited by size
                        ws-tdq                   delimited by size
                        ' does not exist.'       delimited by size
                        ' Terminating.'          delimited by size
                        into ws-send-text
                 end-string
                 perform send-msg
                 perform termination

              end-if


      * write the TDQ data to the TSQ
              perform read-tdq-write-tsq

      * now read first record to reset TSQ pointer and
      * read the name of the TDQ

              exec cics readq ts
                        queue(ws-tsq)
                        item(1)
                        into(ws-ts-record)
                        length(length of ws-ts-record)
                        resp(ws-cics-resp)
              end-exec

      *       string ws-progname              delimited by size
      *              ': TDQ '                 delimited by size
      *              ': first record: '       delimited by size
      *              ws-ts-record             delimited by size
      *              into ws-send-text
      *       end-string
      *       perform send-msg

              move low-values      to dfhcommarea
              move 'TDQ found'     to dfhcommarea

      * ---
      * --- first time through (end) ---
      * ---

      * else there must be data to compare

           else

              perform compare-data

           end-if
           .
       lt-end.
           exit.

      *-----------------------------------------------------------------
       read-tdq-write-tsq section.
       r1.

           exec cics enq
                     resource(ws-tdq)
                     length(length of ws-tdq)
                     resp(ws-cics-resp)
           end-exec

           string ws-progname              delimited by size
                  ': ENQ resource TSQ: '   delimited by size
                  ws-tdq                   delimited by size
                  into ws-send-text
           end-string
           perform send-msg

      * reset TDQ via CLOSE and OPEN
            exec cics set tdqueue(ws-tdq)
                      openstatus(ws-closed)
                      resp(ws-cics-resp)
            end-exec

            if ws-cics-resp = dfhresp(QIDERR) then
               move spaces to ws-send-text
               string 'TD queue '           delimited by size
                       ws-tdq
                      ' not known.'         delimited by size
                    into ws-send-text
               end-string
               perform send-msg
               perform terminate-and-deleteq
            end-if

            if ws-cics-resp not = dfhresp(NORMAL) then
               move spaces to ws-send-text
               move ws-cics-resp to ws-cics-resp-char
               string 'TD queue CLOSE error' delimited by size
                      ' - Resp: '       delimited by size
                      ws-cics-resp-char delimited by size
                    into ws-send-text
               end-string
               perform send-msg
               perform terminate-and-deleteq
            end-if

            exec cics set tdqueue(ws-tdq)
                      openstatus(ws-open)
                      resp(ws-cics-resp)
            end-exec

            if ws-cics-resp not = dfhresp(NORMAL) then
               move spaces to ws-send-text
               move ws-cics-resp to ws-cics-resp-char
               string 'TD queue OPEN error' delimited by size
                      ' - Resp: '       delimited by size
                      ws-cics-resp-char delimited by size
                    into ws-send-text
               end-string
               perform send-msg
               perform terminate-and-deleteq
            end-if
           .

       read-tdq-loop.
           exec cics readq td queue(ws-tdq)
                     into(ws-td-record)
                     length(length of ws-td-record)
                     resp(ws-cics-resp)
                     end-exec.

           if ws-cics-resp not = dfhresp(normal) and
              ws-cics-resp not = dfhresp(qzero) then
              move 'Error in opening input tdq'
                   to ws-send-text
               perform send-msg
               perform terminate-and-deleteq
           end-if

      * write record to tsq
           if ws-cics-resp not = dfhresp(qzero) then

              exec cics writeq ts queue(ws-tsq)
                        from(ws-td-record)
                        length(length of ws-td-record)
                        resp(ws-cics-resp)
                        end-exec

              move spaces to ws-td-record
              go to read-tdq-loop

           end-if
           .
           exec cics deq
                     resource(ws-tdq)
                     length(length of ws-tdq)
                     resp(ws-cics-resp)
           end-exec

           string ws-progname              delimited by size
                  ': DEQ resource TSQ: '   delimited by size
                  ws-tdq                   delimited by size
                  into ws-send-text
           end-string
           perform send-msg

           .
       r-end.
           exit.

      *-----------------------------------------------------------------
       compare-data section.
       cmpdta.
           move eibcalen to  ws-comm-len-rcvd

           string ws-progname                  delimited by size
                  ': data received: '          delimited by size
                  dfhcommarea                  delimited by size
                  ' (eibcalen = '              delimited by size
                  ws-comm-len-rcvd             delimited by size
                  ')'                          delimited by size
                  into ws-send-text
           end-string
           perform send-msg

      * read next record in TSQ
           move spaces to ws-ts-record
           exec cics readq ts queue(ws-tsq)
                     into(ws-ts-record)
                     length(length of ws-ts-record)
                     resp(ws-cics-resp)
           end-exec

           string ws-progname                  delimited by size
                  ': read TSQ record: '        delimited by size
                  ws-ts-record                 delimited by size
                  into ws-send-text
           end-string
           perform send-msg

      * now compare
           if dfhcommarea = ws-ts-record(1:eibcalen) then
               string ws-progname              delimited by size
                      ': Data matched!'        delimited by size
                    into ws-send-text
               end-string
               perform send-msg
               move low-values      to dfhcommarea
               move 'Data matched!' to dfhcommarea
           else
               string ws-progname              delimited by size
                      ': Data did not match'   delimited by size
                    into ws-send-text
               end-string
               perform send-msg
           end-if
           .
       cmpdta-end.
           exit.

      *-----------------------------------------------------------------
       termination section.
       tm.

              exec cics return
              end-exec.

       tm-end.
           exit.

      *-----------------------------------------------------------------
       terminate-and-deleteq section.
       tmdltq.
      * delete TS queue ready for next run, ignoring any errors
              exec cics deleteq ts queue(ws-tsq)
                        resp(ws-cics-resp)
              end-exec

              string ws-progname                  delimited by size
                     ': deleted TSQ '             delimited by size
                     ws-tsq                       delimited by size
                     into ws-send-text
              end-string
              perform send-msg

              exec cics return
              end-exec.

       tmdltq-end.
           exit.

      *-----------------------------------------------------------------

       send-msg.
      *    Comment out as required. Options are to send output
      *    to CSMT, to console (if supported); both or neither.

      *    exec cics writeq td queue('CSMT') from(ws-send-text)
      *              length(80)
      *              nohandle end-exec.

           exec cics write operator text(ws-send-text)
                     textlength(length of ws-send-text)
           end-exec

           move spaces to ws-send-text.

       send-msg-exit.
           exit.

