//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// High-level EPI Utility methods
// Aaron Bell - May 98
//____________________________________________________

package jgtest.util;

import ibm.cics.jgate.epi.*;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

public class HEPIUtil
{
  /*---------------------------------------------------*\
  |  showTerminalStatus - displays Terminal information |
  \*---------------------------------------------------*/
  public static void showTerminalStatus(Terminal term)
  {
    try {
      System.out.println( "    NetName .................. " + term.getNetName() );
      System.out.println( "    DeviceType ............... " + term.getDeviceType() );
      System.out.println( "    Connected to server ...... " + term.getServerName() );
      System.out.println( "    Terminal state ........... " + getTerminalState(term) );
      System.out.println( "    Associated screen ........ " + term.getScreen() );
      System.out.println( "    Current transaction ...... " + term.getTransID() );
      System.out.println( "    ATI state ................ " + (term.queryATI() ? "enabled" : "disabled") );
    }
    catch (EPIException e) {
      System.err.println("EPIException in showTerminalStatus: "+e.getMessage());
    }
    catch (IOException e) {
      System.err.println("IOException in showTerminalStatus: "+e.getMessage());
    }
  }

  /*---------------------------------------------------*\
  |  showFieldStatus - displays Field information       |
  \*---------------------------------------------------*/
  public static void showFieldStatus(Field fld)
  {
    try {
      System.out.println( " Text ..................... " + fld.getText() );
      System.out.println( " Column ................... " + fld.getColumn() );
      System.out.println( " Row ...................... " + fld.getRow() );
      System.out.println( " Total length ............. " + fld.length() );
      System.out.println( " Text length .............. " + fld.textLength() );
      System.out.println( " Input data type .......... " + (fld.inputType() == Field.numeric ? "numeric" : "alphanumeric") );
      System.out.println( " Background colour ........ " + getFieldColor(fld.backgroundColor()) );
      System.out.println( " Foreground colour ........ " + getFieldColor(fld.foregroundColor()) );
      System.out.println( " Highlight ................ " + getFieldHighlight(fld.highlight()) );
      System.out.println( " Intensity ................ " + (fld.intensity() == Field.intense ? "intense" : "normal") );
      System.out.println( " Transparency ............. " + getFieldTrans(fld.transparency()) );
      System.out.println( " Base attribute ........... " + fld.baseAttribute() );
      System.out.println( " Data tag ................. " + (fld.dataTag() == Field.modified ? "modified" : "unmodified") );
      System.out.println( " Protected ................ " + (fld.inputProt() == Field.isprotect ? "protected" : "unprotected") );
    }
    catch (UnsupportedEncodingException e) {
      System.err.println("UnsupportedEncoding in showFieldStatus: "+e.getMessage());
    }
  }

  /*---------------------------------------------------*\
  |  listServers - list servers visible from EPIGateway |
  \*---------------------------------------------------*/
  public static void listServers(EPIGateway epiGate)
  {
    try {
      for (int i=1; i<=epiGate.serverCount(); i++)
        System.out.println("   "+i+". '"+epiGate.serverName(i)+"' Description: '"+epiGate.serverDesc(i)+"'");
    }
    catch (IOException e) {
      System.err.println("IOException in listServers: "+e.getMessage());
    }
    catch (EPIException e) {
      System.err.println("EPIException in listServers: "+e.getMessage());
    }
  }

  /*----------------------------------------------------*\
  |  getServer1 - return name of EPIGateway's 1st server |
  \*----------------------------------------------------*/
  public static String getServer1(EPIGateway epiGate)
  {
    try {
      return epiGate.serverName(1);
    }
    catch (IOException e) {
      System.err.println("IOException in getServer1: "+e.getMessage());
    }
    catch (EPIException e) {
      System.err.println("EPIException in getServer1: "+e.getMessage());
    }
    return "Exception_in_HEPIUtil.getServer1";
  }

  /*---------------------------------------------------*\
  |  getTerminalState - return readable Terminal state  |
  \*---------------------------------------------------*/
  public static String getTerminalState(Terminal term)
  {
    switch (term.getState()) {
      case Terminal.start  : return "start";
      case Terminal.idle   : return "idle";
      case Terminal.client : return "client";
      case Terminal.server : return "server";
      case Terminal.discon : return "discon";
      case Terminal.error  : return "error";
      default : return "unknown!";
    }
  }

  /*---------------------------------------------------*\
  |  getFieldColor - return readable Field colour       |
  \*---------------------------------------------------*/
  public static String getFieldColor(int col)
  {
    switch (col) {
      case Field.black        : return "black";
      case Field.blue         : return "blue";
      case Field.cyan         : return "cyan";
      case Field.darkblue     : return "darkblue";
      case Field.defaultColor : return "defaultColor";
      case Field.gray         : return "gray";
      case Field.green        : return "green";
      case Field.neutral      : return "neutral";
      case Field.orange       : return "orange";
      case Field.paleCyan     : return "paleCyan";
      case Field.paleGreen    : return "paleGreen";
      case Field.pink         : return "pink";
      case Field.purple       : return "purple";
      case Field.red          : return "red";
      case Field.white        : return "white";
      case Field.yellow       : return "yellow";
      default : return "unknown!";
    }
  }

  /*----------------------------------------------------*\
  |  getFieldHighlight - return readable Field Highlight |
  \*----------------------------------------------------*/
  public static String getFieldHighlight(int hl)
  {
    switch (hl) {
      case Field.blinkHlt      : return "blinkHlt";
      case Field.defaultHlt    : return "defaultHlt";
      case Field.intenseHlt    : return "intenseHlt";
      case Field.normalHlt     : return "normalHlt";
      case Field.reverseHlt    : return "reverseHlt";
      case Field.underscoreHlt : return "underscoreHlt";
      default : return "unknown!";
    }
  }

  /*---------------------------------------------------*\
  |  getFieldTrans - return readable Field Transparency |
  \*---------------------------------------------------*/
  public static String getFieldTrans(int t)
  {
    switch (t) {
      case Field.defaultTran : return "defaultTran";
      case Field.opaqueTran  : return "opaqueTran";
      case Field.orTran      : return "orTran";
      case Field.xorTran     : return "xorTran";
      default : return "unknown!";
    }
  }
}
