//3     Input Parameters for JGTEST Application Suite
//
//p     The input parameters are unique across the whole application suite
//      although not all are applicable to individual applications.
//
//p     To list application input parameters, and the default settings, issue
//      the name of the application, followed by -?
//p     For example:
//x     java jgtest.eci.ecij123 -?
//
//p     will display:
//x
//      ecij123 - Three Simple ECI Calls
//             -j     JGate                    -> hollings
//             -r     JGate port               -> 2006
//             -list  list systems only        -> false
//             -s     CICS server              -> (not specified)
//             -a     application              -> SRVTIME
//             -i     # iterations             -> 1
//             -u     userid                   -> SYSAD
//             -p     password                 -> SYSAD
//             -d     delay between iterations -> 0 seconds
//             -x     exit on ECI error        -> false
//             -!     debug                    -> false
//             -?     Display settings only
//
//      Usage: java ecij123 <parameters>
//        e.g. java ecij123 -s AlsKix -i 5
//
//p     The current list contains:
//
//D     -!
//d     To enable application debug. The trace will include entries in the
//      code that use the T.ln and T.ex class methods.
//
//D     -a
//d     Server application name. The default is SRVTIME.
//
//D     -calls
//d     Number of ECI asynchronous calls within an iteration (see ECI
//      application ascj001)
//
//D     -clear
//d     Clear outstanding ECI asynchronous replies before issuing any further
//      ECI calls (see ECI application asmj002)
//
//D     -clisec
//d     Use client security class com.ibm.ctg.security.clientDesign
//
//D     -cmlen
//d     The initial commarea length to be sent to the CICS server
//
//D     -eci
//d     Call eci_list_systems (see gateway application exijs01)
//
//D     -epi
//d     Call epi_list_systems (see gateway application exijs01)
//
//D     -d
//d     Delay between calls or iterations, in seconds.
//
//D     -file
//d     Name of shared NLS data file (see NLS application ecijnls)
//
//D     -help
//d     Display help and default settings
//
//D     -hold
//d     Up to three digits to request the CICS server program, dlytime,
//      to issue an EXEC CICS DELAY for 'nnn' seconds (see ECI application
//      dlyj001)
//
//D     -iterations
//d     Number of iterations the application should run.
//
//D     -j
//d     The name of the JGate. This can be a hostname, TCP/IP address, and
//      can be prefixed with, for example, http: for protocol testing.
//      If not specified, the default is the local hostname.
//
//D     -list
//d     Issue an ECI_LIST_SYSTEMS or EPI_LIST_SYSTEMS to list available
//      systems. The application stops after listing the details.
//
//D     -luw
//d     Number of concurrent logical units of work (LUW)
//
//D     -model
//d     The EPI terminal model name
//
//D     -p
//d     Password passed on ECI calls. The default is SYSAD.
//
//D     -r
//d     The JGate listening port. The default is 2006. Not applicable if
//      the Local Java Gateway is being used (ie -j local:)
//
//D     -s1
//d     Name of first CICS server to send the ECI request to (see ECI
//      application synjms3). The default is the first server named
//      in the CICS Client CICSCLI.INI file.
//
//D     -s2
//d     Name of second CICS server to send the ECI request to (see ECI
//      application synjms3)
//
//D     -s3
//d     Name of third CICS server to send the ECI request to (see ECI
//      application synjms3)
//
//D     -s
//d     Name of CICS server to send the ECI/EPI request to. The default
//      is the first server named in the CICS Client CICSCLI.INI file.
//      CICSMRO can be used if the CICS Gateway for Java and CICS for OS/2
//      reside on the same workstation. For CICS Gateway for Java (MVS)
//      the CICS server is pointed to by the variable DFHJVSYSTEM_00 by
//      default.
//
//D     -srvsec
//d     Use server security class com.ibm.ctg.security.serverDesign
//
//D     -tran
//d     Alternative transid for CICS mirror when used with ECI_SYNC_TPN
//      and ECI_ASYNC_TPN calls
//
//D     -u
//d     Userid passed on ECI calls. Default is SYSAD.
//
//D     -verbose
//d     Displays extra information during runtime
//
//D     -wait
//d     Use the eci_get_reply_wait option
//
//D     -x
//d     Exit program upon first ECI error
//
//D     -+
//d     Increment commarea length by 1 (see ECI application synj0c3)
//
//D     -2sec
//d     Two second delay between asynchronous calls (see ECI application
//      asmj002)
//
//D     -?
//d     Display help and default settings
//
//p
//      History log:
//u
//l     Written : Apr 98 - Alan Hollingshead
//l     Updated : Jun 98 - Aaron Bell - added bVerbose, s*Security,
//      processParms(String[] args)
//

package jgtest.util;

import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.io.*;
import java.net.*;
import java.text.*;

public class inputParms
{
   public static String sCalls          = "1";
   public static String sCommLen        = "18";
   public static String sDelay          = "0";
   public static String sFilename       = "NLS-US.DAT";
   public static String sHold           = "000";
   public static String sIterations     = "1";
   public static String sJGate          = "" ;
   public static String sModel          = "DFLT";
   public static String sNumLUWs        = "1";
   public static String sServer         = "(not specified)" ;
   public static String sServer1        = "(not specified)" ;
   public static String sServer2        = "(not specified)" ;
   public static String sServer3        = "(not specified)" ;
   public static String sClientSecurity = "(not specified)" ;
   public static String sServerSecurity = "(not specified)" ;
   public static String sPassword       = "SYSAD" ;
   public static String sPort           = "2006" ;
   public static String sProgname       = "SRVTIME" ;
   public static String sTransid        = "CPMI" ;
   public static String sUserid         = "SYSAD" ;

   public static boolean bAsyncDelay    = false ;
   public static boolean bCallECI       = false ;
   public static boolean bCallEPI       = false ;
   public static boolean bClearReplies  = false ;
   public static boolean bDebug         = false ;
   public static boolean bExitNow       = false ;
   public static boolean bHelp          = false ;
   public static boolean bIncrementComm = false ;
   public static boolean bListSystems   = false ;
   public static boolean bWait          = false ;
   public static boolean bVerbose       = false ;

   public static int     iCalls         = 1;
   public static int     iCommLen       = 18;
   public static int     iDelay         = 0;
   public static int     iIterations    = 1;
   public static int     iNumLUWs       = 1;
   public static int     iPort          = 2006;

   public inputParms()
   {
     // create with defaults only
     processParms(new String[0]);
   }

   public inputParms(String[] args)
   {
     // create and process the provided arguments
     processParms(args);
   }

   public void processParms(String[] args)
   {
      // Parse arguments passed in on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            //
            // -! = set Java application debug on
            //
            if (args[loop].equalsIgnoreCase("-!")) {
               bDebug=true ;
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            //
            // -a = CICS application
            //
            else if (args[loop].equalsIgnoreCase("-a")) {
               sProgname = args[loop+1] ;
               ++loop ;
            }
            //
            // -calls = calls per iteration (ascj001)
            //
            else if (args[loop].equalsIgnoreCase("-calls")) {
               sCalls = args[loop+1];
               // convert to integer
               iCalls = Integer.parseInt(sCalls);
            }
            //
            // -clear = clear outstanding replies first
            //
            else if (args[loop].equalsIgnoreCase("-clear")) {
               bClearReplies = true;
            }
            //
            // -clisec = Client security
            //
            else if (args[loop].equalsIgnoreCase("-clisec")) {
               sClientSecurity = args[loop+1] ;
               ++loop ;
            }
            //
            // -cmlen = initial commarea length
            //
            else if (args[loop].equalsIgnoreCase("-cmlen")) {
               sCommLen = args[loop+1];
               ++loop;
               // convert to integer
               iCommLen = Integer.parseInt(sCommLen);
            }
            //
            // -eci = call eci_list_systems (exijs01)
            //
            else if (args[loop].equalsIgnoreCase("-eci")) {
               bCallECI = true;
            }
            //
            // -epi = call epi_list_systems (exijs01)
            //
            else if (args[loop].equalsIgnoreCase("-epi")) {
               bCallEPI = true;
            }
            //
            // -d = delay in seconds between each ECI call unless
            //      otherwise specified on help panel
            //
            else if (args[loop].equalsIgnoreCase("-d")) {
               sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               iDelay = Integer.parseInt(sDelay) * 1000;
            }
            //
            // -file = data file name
            //
            else if (args[loop].equalsIgnoreCase("-file")) {
               sFilename = args[loop+1] ;
               ++loop ;
            }
            //
            // -hold = iterations
            //
            else if (args[loop].equalsIgnoreCase("-hold")) {
               sHold = args[loop+1] ;
               ++loop ;
               // convert to integer
               int iHold = Integer.parseInt(sHold);
               // pad -hold value with leading zeros if necessary
               NumberFormat nf = NumberFormat.getInstance();
               nf.setMinimumIntegerDigits(3);
               // call format to pad numbers
               sHold = nf.format(iHold);
            }
            //
            // -i = iterations
            //
            else if (args[loop].equalsIgnoreCase("-i")) {
               sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               iIterations = Integer.parseInt(sIterations);
            }
            //
            // -j = JGate
            //
            else if (args[loop].equalsIgnoreCase("-j")) {
               sJGate = args[loop+1] ;
               ++loop ;
            }
            //
            // -list = list systems available
            //
            else if (args[loop].equalsIgnoreCase("-list")) {
               bListSystems = true;
            }
            //
            // -luw = number of logical units of work
            //
            else if (args[loop].equalsIgnoreCase("-luw")) {
               sNumLUWs = args[loop+1] ;
               ++loop ;
               // convert to integer
               iNumLUWs = Integer.parseInt(sNumLUWs);
            }
            //
            // -model = model / device type
            //
            else if (args[loop].equalsIgnoreCase("-model")) {
               sModel = args[loop+1] ;
               ++loop ;
            }
            //
            // -p = CICS password
            //
            else if (args[loop].equalsIgnoreCase("-p")) {
               sPassword = args[loop+1] ;
               ++loop ;
            }
            //
            // -r = JGate port
            //
            else if (args[loop].equalsIgnoreCase("-r")) {
               sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               iPort = Integer.parseInt(sPort);
            }
            //
            // -s = CICS server
            //
            else if (args[loop].equalsIgnoreCase("-s")) {
               sServer = args[loop+1] ;
               ++loop ;
            }
            //
            // -s1 = CICS server1
            //
            else if (args[loop].equalsIgnoreCase("-s1")) {
               sServer1 = args[loop+1] ;
               ++loop ;
            }
            //
            // -s2 = CICS server2
            //
            else if (args[loop].equalsIgnoreCase("-s2")) {
               sServer2 = args[loop+1] ;
               ++loop ;
            }
            //
            // -s3 = CICS server3
            //
            else if (args[loop].equalsIgnoreCase("-s3")) {
               sServer3 = args[loop+1] ;
               ++loop ;
            }
            //
            // -srvsec = Server security
            //
            else if (args[loop].equalsIgnoreCase("-srvsec")) {
               sServerSecurity = args[loop+1] ;
               ++loop ;
            }
            //
            // -tran = alternative transid for mirror
            //
            else if (args[loop].equalsIgnoreCase("-tran")) {
               sTransid = args[loop+1] ;
               ++loop ;
            }
            //
            // -u = CICS userid
            //
            else if (args[loop].equalsIgnoreCase("-u")) {
               sUserid = args[loop+1] ;
               ++loop ;
            }
            //
            // -wait = eci_get_reply_wait requested
            //
            else if (args[loop].equalsIgnoreCase("-wait")) {
               bWait = true;
            }
            //
            // -verbose = verbose output
            //
            else if (args[loop].equalsIgnoreCase("-verbose")) {
               bVerbose = true;
            }
            //
            // -x = Exit program upon first ECI error
            //
            else if (args[loop].equalsIgnoreCase("-x")) {
               bExitNow = true;
            }
            //
            // -+ = increment commarea length
            //
            else if (args[loop].equalsIgnoreCase("-+")) {
               bIncrementComm = true;
            }
            //
            // -2sec = Two second delay between async calls
            //
            else if (args[loop].equalsIgnoreCase("-2sec")) {
               bAsyncDelay = true;
            }
            //
            // -? or -help = Help!
            //
            else if ((args[loop].equalsIgnoreCase("-?")) ||
                     (args[loop].equalsIgnoreCase("help"))) {
               bHelp = true;
            }
         } // end-for loop
      } // end-if-args

      // ---------------------------------------------------------------------
      // If JGate not specified then use local hostname.
      // May require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.
      // ---------------------------------------------------------------------

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            System.out.println("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

   }
}
